@@ -26,12 +26,16 @@
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
+#ifndef _WIN32_WCE
 #include <errno.h>
+#include <unistd.h>
+#endif /*_WIN32_WCE*/
 #include <limits.h>
 #include <ctype.h>
-#include <unistd.h>
 #include <linphonecore.h>
 #include "linphonec.h"
+#include "private.h"
+#include "lpconfig.h"
 
 #ifndef WIN32
 #include <sys/wait.h>
@@ -49,6 +53,7 @@ extern char *lpc_strip_blanks(char *input);
 static int lpc_cmd_help(LinphoneCore *, char *);
 static int lpc_cmd_proxy(LinphoneCore *, char *);
 static int lpc_cmd_call(LinphoneCore *, char *);
+static int lpc_cmd_chat(LinphoneCore *, char *);
 static int lpc_cmd_answer(LinphoneCore *, char *);
 static int lpc_cmd_autoanswer(LinphoneCore *, char *);
 static int lpc_cmd_terminate(LinphoneCore *, char *);
@@ -61,6 +66,8 @@ static int lpc_cmd_stun(LinphoneCore *, char *);
 static int lpc_cmd_firewall(LinphoneCore *, char *);
 static int lpc_cmd_friend(LinphoneCore *, char*);
 static int lpc_cmd_soundcard(LinphoneCore *, char *);
+static int lpc_cmd_webcam(LinphoneCore *, char *);
+static int lpc_cmd_staticpic(LinphoneCore *, char *);
 static int lpc_cmd_play(LinphoneCore *, char *);
 static int lpc_cmd_record(LinphoneCore *, char *);
 static int lpc_cmd_register(LinphoneCore *, char *);
@@ -69,6 +76,8 @@ static int lpc_cmd_duration(LinphoneCore *lc, char *args);
 static int lpc_cmd_status(LinphoneCore *lc, char *args);
 static int lpc_cmd_ports(LinphoneCore *lc, char *args);
 static int lpc_cmd_speak(LinphoneCore *lc, char *args);
+static int lpc_cmd_codec(LinphoneCore *lc, char *args);
+static int lpc_cmd_echocancellation(LinphoneCore *lc, char *args);
 
 /* Command handler helpers */
 static void linphonec_proxy_add(LinphoneCore *lc);
@@ -85,6 +94,10 @@ static int linphonec_friend_call(LinphoneCore *lc, unsigned int num);
 static int linphonec_friend_add(LinphoneCore *lc, const char *name, const char *addr);
 #endif
 static int linphonec_friend_delete(LinphoneCore *lc, int num);
+static int linphonec_friend_delete(LinphoneCore *lc, int num);
+static void linphonec_codec_list(LinphoneCore *lc);
+static void linphonec_codec_enable(LinphoneCore *lc, int index);
+static void linphonec_codec_disable(LinphoneCore *lc, int index);
 
 
 
@@ -107,9 +120,13 @@ void linphonec_out(const char *fmt,...);
 LPC_COMMAND commands[] = {
 	{ "help", lpc_cmd_help, "Print commands help", NULL },
 	{ "call", lpc_cmd_call, "Call a SIP uri",
-		"'call <sip-url>' or 'c <sip-url>' "
+		"'call <sip-url>' "
 		": initiate a call to the specified destination."
 		},
+	{ "chat", lpc_cmd_chat, "Chat with a SIP uri",
+		"'chat <sip-url> \"message\"' "
+		": send a chat message \"message\" to the specified destination."
+		},
 	{ "terminate", lpc_cmd_terminate, "Terminate the current call",
 		NULL },
 	{ "answer", lpc_cmd_answer, "Answer a call",
@@ -134,6 +151,13 @@ LPC_COMMAND commands[] = {
 		"'soundcard use <index>' : select a sound device.\n"
 		"'soundcard use files' : use .wav files instead of soundcard\n"
 	},
+	{ "webcam", lpc_cmd_webcam, "Manage webcams",
+		"'webcam list' : list all known devices.\n"
+		"'webcam use <index>' : select a video device.\n"
+	},
+	{ "staticpic", lpc_cmd_staticpic, "Manage static pictures when nowebcam",
+		"'staticpic set' : Set path to picture that should be used.\n"
+	},
 	{ "ipv6", lpc_cmd_ipv6, "Use IPV6",
 		"'ipv6 status' : show ipv6 usage status.\n"
 		"'ipv6 enable' : enable the use of the ipv6 network.\n"
@@ -191,6 +215,14 @@ LPC_COMMAND commands[] = {
 			"'speak <voice name> <sentence>'	: speak a text using the specified espeak voice.\n"
 			"Example for english voice: 'speak default Hello my friend !'"
 	},
+    { "codec", lpc_cmd_codec, "Codec configuration",
+            "'codec list' : list codecs\n"  
+            "'codec enable <index>' : enable available codec\n"  
+            "'codec disable <index>' : disable codecs" }, 
+    { "ec", lpc_cmd_echocancellation, "Echo cancellation",
+            "'ec on [<delay>] [<tail>] [<framesize>]' : turn EC on with given delay, tail length and framesize\n"
+            "'ec off' : turn echo cancellation (EC) off\n"
+            "'ec show' : show EC status" },
 	{ (char *)NULL, (lpc_cmd_handler)NULL, (char *)NULL, (char *)NULL }
 };
 
@@ -282,7 +314,7 @@ linphonec_command_generator(const char *text, int state)
 
 		if (strncmp(name, text, len) == 0)
 		{
-			return strdup(name);
+			return ortp_strdup(name);
 		}
 	}
 
@@ -361,6 +393,35 @@ lpc_cmd_call(LinphoneCore *lc, char *args)
 	return 1;
 }
 
+static int
+lpc_cmd_chat(LinphoneCore *lc, char *args)
+{
+	char *arg1 = args;
+	char *arg2 = NULL;
+	char *ptr = args;
+
+	if (!args) return 0;
+
+	/* Isolate first and second arg */
+	while(*ptr && !isspace(*ptr)) ++ptr;
+	if ( *ptr )
+	{
+		*ptr='\0';
+		arg2=ptr+1;
+		while(*arg2 && isspace(*arg2)) ++arg2;
+	}
+	else
+	{
+		/* missing one parameter */
+		return 0;
+	}
+	LinphoneChatRoom *cr = linphone_core_create_chat_room(lc,arg1);
+	linphone_chat_room_send_message(cr,arg2);
+	linphone_chat_room_destroy(cr);
+
+	return 1;
+}
+
 const char *linphonec_get_callee(){
 	return callee_name;
 }
@@ -577,10 +638,12 @@ lpc_cmd_friend(LinphoneCore *lc, char *args)
 		args+=4;
 		if ( ! *args ) return 0;
 		friend_num = strtol(args, NULL, 10);
+#ifndef _WIN32_WCE		
 		if ( errno == ERANGE ) {
 			linphonec_out("Invalid friend number\n");
 			return 0;
 		}
+#endif /*_WIN32_WCE*/
 		linphonec_friend_call(lc, friend_num);
 		return 1;
 	}
@@ -597,10 +660,12 @@ lpc_cmd_friend(LinphoneCore *lc, char *args)
 		else
 		{
 			friend_num = strtol(args, NULL, 10);
+#ifndef _WIN32_WCE		
 			if ( errno == ERANGE ) {
 				linphonec_out("Invalid friend number\n");
 				return 0;
 			}
+#endif /*_WIN32_WCE*/
 		}
 		linphonec_friend_delete(lc, friend_num);
 		return 1;
@@ -748,7 +813,7 @@ lpc_cmd_proxy(LinphoneCore *lc, char *args)
 static int
 lpc_cmd_call_logs(LinphoneCore *lc, char *args)
 {
-	MSList *elem=linphone_core_get_call_logs(lc);
+	const MSList *elem=linphone_core_get_call_logs(lc);
 	for (;elem!=NULL;elem=ms_list_next(elem))
 	{
 		LinphoneCallLog *cl=(LinphoneCallLog*)elem->data;
@@ -926,6 +991,80 @@ static int lpc_cmd_soundcard(LinphoneCore *lc, char *args)
 	return 0; /* syntax error */
 }
 
+static int lpc_cmd_webcam(LinphoneCore *lc, char *args)
+{
+	int i, index;
+	const char **dev;
+	char *arg1 = args;
+	char *arg2 = NULL;
+	char *ptr = args;
+
+	if (!args) return 0; /* syntax error */
+
+	/* Isolate first and second arg */
+	while(*ptr && !isspace(*ptr)) ++ptr;
+	if ( *ptr )
+	{
+		*ptr='\0';
+		arg2=ptr+1;
+		while(*arg2 && isspace(*arg2)) ++arg2;
+	}
+
+	if (strcmp(arg1, "list")==0)
+	{
+		dev=linphone_core_get_video_devices(lc);
+		for(i=0; dev[i]!=NULL; ++i){
+			linphonec_out("%i: %s\n",i,dev[i]);
+		}
+		return 1;
+	}
+
+	if (strcmp(arg1, "use")==0 && arg2)
+	{
+		dev=linphone_core_get_video_devices(lc);
+		index=atoi(arg2); /* FIXME: handle not-a-number */
+		for(i=0;dev[i]!=NULL;i++)
+		{
+			if (i!=index) continue;
+
+			linphone_core_set_video_device(lc, dev[i]);
+			linphonec_out("Using video device %s\n",dev[i]);
+			return 1;
+		}
+		linphonec_out("No such video device\n");
+		return 1;
+	}
+	return 0; /* syntax error */
+}
+
+static int
+lpc_cmd_staticpic(LinphoneCore *lc, char *args)
+{
+	char *arg1 = args;
+	char *arg2 = NULL;
+	char *ptr = args;
+
+	if (!args) return 0;  /* Syntax error */
+
+	/* Isolate first and second arg */
+	while(*ptr && !isspace(*ptr)) ++ptr;
+	if ( *ptr )
+	{
+		*ptr='\0';
+		arg2=ptr+1;
+		while(*arg2 && isspace(*arg2)) ++arg2;
+	}
+
+	if (strcmp(arg1, "set")==0 && arg2) {
+		linphone_core_set_static_picture(lc, arg2);
+		return 1;
+	}
+
+	return 0; /* Syntax error */
+}
+
+
+
 /***************************************************************************
  *
  *  Commands helper functions
@@ -1233,12 +1372,13 @@ linphonec_proxy_use(LinphoneCore *lc, int index)
 static void
 linphonec_friend_display(LinphoneFriend *fr)
 {
-	char *name = linphone_friend_get_name(fr);
-	char *addr = linphone_friend_get_addr(fr);
-	//char *url = linphone_friend_get_url(fr);
-
-	linphonec_out("name: %s\n", name);
-	linphonec_out("address: %s\n", addr);
+	LinphoneAddress *uri=linphone_address_clone(linphone_friend_get_address(fr));
+	char *str;
+	
+	linphonec_out("name: %s\n", linphone_address_get_display_name(uri));
+	linphone_address_set_display_name(uri,NULL);
+	str=linphone_address_as_string(uri);
+	linphonec_out("address: %s\n", str);
 }
 
 static int
@@ -1256,8 +1396,9 @@ linphonec_friend_list(LinphoneCore *lc, char *pat)
 	for(n=0; friend!=NULL; friend=ms_list_next(friend), ++n )
 	{
 		if ( pat ) {
-			char *name = linphone_friend_get_name(friend->data);
-			if ( ! strstr(name, pat) ) continue;
+			const char *name = linphone_address_get_display_name(
+			    linphone_friend_get_address((LinphoneFriend*)friend->data));
+			if (name && ! strstr(name, pat) ) continue;
 		}
 		linphonec_out("****** Friend %i *******\n",n);
 		linphonec_friend_display((LinphoneFriend*)friend->data);
@@ -1277,8 +1418,11 @@ linphonec_friend_call(LinphoneCore *lc, unsigned int num)
 	{
 		if ( n == num )
 		{
-			addr = linphone_friend_get_addr(friend->data);
-			return lpc_cmd_call(lc, addr);
+			int ret;
+			addr = linphone_address_as_string(linphone_friend_get_address((LinphoneFriend*)friend->data));
+			ret=lpc_cmd_call(lc, addr);
+			ms_free(addr);
+			return ret;
 		}
 	}
 	linphonec_out("No such friend %u\n", num);
@@ -1341,6 +1485,26 @@ static int lpc_cmd_register(LinphoneCore *lc, char *args){
 	char passwd[512];
 	LinphoneProxyConfig *cfg;
 	const MSList *elem;
+    
+	if (!args)
+    	{
+    		/* it means that you want to register the default proxy */
+    		LinphoneProxyConfig *cfg=NULL;
+    		linphone_core_get_default_proxy(lc,&cfg);
+    		if (cfg)
+    		{
+    			if(!linphone_proxy_config_is_registered(cfg)) {
+				linphone_proxy_config_enable_register(cfg,TRUE);
+				linphone_proxy_config_done(cfg);
+			}else{
+				linphonec_out("default proxy already registered\n");
+			}
+    		}else{
+    			linphonec_out("we do not have a default proxy\n");
+    			return 0;
+    		}
+    		return 1;
+    	}
 	passwd[0]=proxy[0]=identity[0]='\0';
 	sscanf(args,"%s %s %s",identity,proxy,passwd);
 	if (proxy[0]=='\0' || identity[0]=='\0'){
@@ -1348,16 +1512,16 @@ static int lpc_cmd_register(LinphoneCore *lc, char *args){
 		return 1;
 	}
 	if (passwd[0]!='\0'){
-		osip_from_t *from;
+		LinphoneAddress *from;
 		LinphoneAuthInfo *info;
-		osip_from_init(&from);
-		if (osip_from_parse(from,identity)==0){
+		if ((from=linphone_address_new(identity))!=NULL){
 			char realm[128];
-			snprintf(realm,sizeof(realm)-1,"\"%s\"",from->url->host);
-			info=linphone_auth_info_new(from->url->username,NULL,passwd,NULL,realm);
+			snprintf(realm,sizeof(realm)-1,"\"%s\"",linphone_address_get_domain(from));
+			info=linphone_auth_info_new(linphone_address_get_username(from),NULL,passwd,NULL,NULL);
 			linphone_core_add_auth_info(lc,info);
+			linphone_address_destroy(from);
+			linphone_auth_info_destroy(info);
 		}
-		osip_from_free(from);
 	}
 	elem=linphone_core_get_proxy_config_list(lc);
 	if (elem) {
@@ -1401,9 +1565,9 @@ static int lpc_cmd_duration(LinphoneCore *lc, char *args){
 
 static int lpc_cmd_status(LinphoneCore *lc, char *args)
 {
-	if ( ! args ) return 0;
-
 	LinphoneProxyConfig *cfg;
+	
+	if ( ! args ) return 0;
 	linphone_core_get_default_proxy(lc,&cfg);
 	if (strstr(args,"register"))
 	{
@@ -1488,6 +1652,8 @@ static int lpc_cmd_speak(LinphoneCore *lc, char *args){
 	char *wavfile;
 	int status;
 	FILE *file;
+	
+    if (!args) return 0;
 	memset(voice,0,sizeof(voice));
 	sscanf(args,"%s63",voice);
 	sentence=args+strlen(voice);
@@ -1511,6 +1677,149 @@ static int lpc_cmd_speak(LinphoneCore *lc, char *args){
 	return 1;
 }
 
+static int lpc_cmd_codec(LinphoneCore *lc, char *args){
+	char *arg1 = args;
+	char *arg2 = NULL;
+	char *ptr = args;
+
+	if (!args) return 0;
+
+	/* Isolate first and second arg */
+	while(*ptr && !isspace(*ptr)) ++ptr;
+	if ( *ptr )
+	{
+		*ptr='\0';
+		arg2=ptr+1;
+		while(*arg2 && isspace(*arg2)) ++arg2;
+	}
+
+	if (strcmp(arg1,"enable")==0)
+	{
+#ifdef HAVE_READLINE
+		rl_inhibit_completion=1;
+#endif
+        if (!strcmp(arg2,"all")) linphonec_codec_enable(lc,-1);
+        else linphonec_codec_enable(lc,atoi(arg2));
+#ifdef HAVE_READLINE
+		rl_inhibit_completion=0;
+#endif
+	}
+	else if (strcmp(arg1,"list")==0)
+	{
+		linphonec_codec_list(lc);
+	}
+	else if (strcmp(arg1,"disable")==0)
+	{
+        if (!strcmp(arg2,"all")) linphonec_codec_disable(lc,-1);
+        else linphonec_codec_disable(lc,atoi(arg2));
+	}
+	else
+	{
+		return 0; /* syntax error */
+	}
+
+	return 1;
+}
+
+static void linphonec_codec_list(LinphoneCore *lc){
+	PayloadType *pt;
+    codecs_config_t *config=&lc->codecs_conf;
+	int index=0;
+	MSList *node;
+	for(node=config->audio_codecs;node!=NULL;node=ms_list_next(node)){
+		pt=(PayloadType*)(node->data);
+        linphonec_out("%2d: %s (%d) %s\n", index, pt->mime_type, pt->clock_rate, 
+		    linphone_core_payload_type_enabled(lc,pt) ? "enabled" : "disabled");
+		index++;
+	}
+}
+
+static void linphonec_codec_enable(LinphoneCore *lc, int sel_index){
+	PayloadType *pt;
+    codecs_config_t *config=&lc->codecs_conf;
+	int index=0;
+	MSList *node;
+    for(node=config->audio_codecs;node!=NULL;node=ms_list_next(node)){
+        if (index == sel_index || sel_index == -1) {
+		    pt=(PayloadType*)(node->data);
+            pt->flags|=PAYLOAD_TYPE_ENABLED;
+            linphonec_out("%2d: %s (%d) %s\n", index, pt->mime_type, pt->clock_rate, "enabled");
+        }
+		index++;
+	}
+}
+
+static void linphonec_codec_disable(LinphoneCore *lc, int sel_index){
+	PayloadType *pt;
+    codecs_config_t *config=&lc->codecs_conf;
+	int index=0;
+	MSList *node;
+	for(node=config->audio_codecs;node!=NULL;node=ms_list_next(node)){
+        if (index == sel_index || sel_index == -1) {
+    		pt=(PayloadType*)(node->data);
+            pt->flags&=~PAYLOAD_TYPE_ENABLED;
+            linphonec_out("%2d: %s (%d) %s\n", index, pt->mime_type, pt->clock_rate, "disabled");
+        }
+		index++;
+	}
+}
+
+static int lpc_cmd_echocancellation(LinphoneCore *lc, char *args){
+	char *arg1 = args;
+	char *arg2 = NULL;
+	char *ptr = args;
+
+	if (!args) return 0;
+
+	/* Isolate first and second arg */
+	while(*ptr && !isspace(*ptr)) ++ptr;
+	if ( *ptr )
+	{
+		*ptr='\0';
+		arg2=ptr+1;
+		while(*arg2 && isspace(*arg2)) ++arg2;
+	}
+
+	if (strcmp(arg1,"on")==0){
+        int delay, tail_len, frame_size;
+        int n;
+
+        linphone_core_enable_echo_cancellation(lc,1);
+
+        if (arg2 != 0) {
+            n = sscanf(arg2, "%d %d %d", &delay, &tail_len, &frame_size);
+
+            if (n == 1) {   
+                lp_config_set_int(lc->config,"sound","ec_delay",delay);
+            }
+            else if (n == 2) {
+                lp_config_set_int(lc->config,"sound","ec_delay",delay);
+                lp_config_set_int(lc->config,"sound","ec_tail_len",tail_len);
+            }
+            else if (n == 3) {
+                lp_config_set_int(lc->config,"sound","ec_delay",delay);
+                lp_config_set_int(lc->config,"sound","ec_tail_len",tail_len);
+                lp_config_set_int(lc->config,"sound","ec_framesize",frame_size);
+            }
+        }
+    }
+    else if (strcmp(arg1,"off")==0){
+        linphone_core_enable_echo_cancellation(lc,0);
+    }
+    else if (strcmp(arg1,"show")==0){
+        linphonec_out("echo cancellation is %s; delay %d, tail length %d, frame size %d\n", 
+            linphone_core_echo_cancellation_enabled(lc) ? "on" : "off",
+            lp_config_get_int(lc->config,"sound","ec_delay",0),
+            lp_config_get_int(lc->config,"sound","ec_tail_len",0),
+            lp_config_get_int(lc->config,"sound","ec_framesize",0));        
+    }
+    else {
+        return 0;
+    }
+
+    return 1;
+}
+
 /***************************************************************************
  *
  *  Command table management funx
