@@ -19,8 +19,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #ifndef LINPHONECORE_H
 #define LINPHONECORE_H
 
-
-#include <osipparser2/osip_message.h>
 #include "ortp/ortp.h"
 #include "ortp/payloadtype.h"
 #include "mediastreamer2/mscommon.h"
@@ -41,140 +39,104 @@ extern "C" {
 
 struct _MSSndCard;
 struct _LinphoneCore;
-
-bool_t payload_type_enabled(struct _PayloadType *pt);
-void payload_type_set_enable(struct _PayloadType *pt,int value);
-const char *payload_type_get_description(struct _PayloadType *pt);
-int payload_type_get_bitrate(PayloadType *pt);
-const char *payload_type_get_mime(PayloadType *pt);
-int payload_type_get_rate(PayloadType *pt);
-
+struct SalOp;
 
 struct _LpConfig;
 
-typedef struct sip_config
-{
-	char *contact;
-	char *guessed_contact;
-	int sip_port;
-	MSList *proxies;
-	MSList *deleted_proxies;
-	int inc_timeout;	/*timeout after an un-answered incoming call is rejected*/
-	bool_t use_info;
-	bool_t guess_hostname;
-	bool_t loopback_only;
-	bool_t ipv6_enabled;
-	bool_t sdp_200_ack;
-	bool_t only_one_codec; /*in SDP answers*/
-	bool_t register_only_when_network_is_up;
-} sip_config_t;
 
-typedef struct rtp_config
-{
-	int audio_rtp_port;
-	int video_rtp_port;
-	int audio_jitt_comp;  /*jitter compensation*/
-	int video_jitt_comp;  /*jitter compensation*/
-	int nortp_timeout;
-}rtp_config_t;
+/**
+ * Object that represents a SIP address.
+ *
+ * The LinphoneAddress is an opaque object to represents SIP addresses, ie
+ * the content of SIP's 'from' and 'to' headers.
+ * A SIP address is made of display name, username, domain name, port, and various
+ * uri headers (such as tags). It looks like 'Alice <sip:alice@example.net>'.
+ * The LinphoneAddress has methods to extract and manipulate all parts of the address.
+ * When some part of the address (for example the username) is empty, the accessor methods
+ * return NULL.
+ * 
+ * @ingroup linphone_address
+ * @var LinphoneAddress
+ */
+typedef struct SalAddress LinphoneAddress;
 
+LinphoneAddress * linphone_address_new(const char *uri);
+LinphoneAddress * linphone_address_clone(const LinphoneAddress *uri);
+const char *linphone_address_get_scheme(const LinphoneAddress *u);
+const char *linphone_address_get_display_name(const LinphoneAddress* u);
+const char *linphone_address_get_username(const LinphoneAddress *u);
+const char *linphone_address_get_domain(const LinphoneAddress *u);
+void linphone_address_set_display_name(LinphoneAddress *u, const char *display_name);
+void linphone_address_set_username(LinphoneAddress *uri, const char *username);
+void linphone_address_set_domain(LinphoneAddress *uri, const char *host);
+void linphone_address_set_port(LinphoneAddress *uri, const char *port);
+void linphone_address_set_port_int(LinphoneAddress *uri, int port);
+/*remove tags, params etc... so that it is displayable to the user*/
+void linphone_address_clean(LinphoneAddress *uri);
+char *linphone_address_as_string(const LinphoneAddress *u);
+char *linphone_address_as_string_uri_only(const LinphoneAddress *u);
+void linphone_address_destroy(LinphoneAddress *u);
 
-
-typedef struct net_config
-{
-	char *nat_address;
-	char *stun_server;
-	char *relay;
-	int download_bw;
-	int upload_bw;
-	int firewall_policy;
-	int mtu;
-	bool_t nat_sdp_only;
-}net_config_t;
-
-
-typedef struct sound_config
-{
-	struct _MSSndCard * ring_sndcard;	/* the playback sndcard currently used */
-	struct _MSSndCard * play_sndcard;	/* the playback sndcard currently used */
-	struct _MSSndCard * capt_sndcard; /* the capture sndcard currently used */
-	const char **cards;
-	int latency;	/* latency in samples of the current used sound device */
-	char rec_lev;
-	char play_lev;
-	char ring_lev;
-	char source;
-	char *local_ring;
-	char *remote_ring;
-	bool_t ec;
-	bool_t ea;
-	bool_t agc;
-} sound_config_t;
-
-typedef struct codecs_config
-{
-	MSList *audio_codecs;  /* list of audio codecs in order of preference*/
-	MSList *video_codecs;	/* for later use*/
-}codecs_config_t;
-
-typedef struct video_config{
-	struct _MSWebCam *device;
-	const char **cams;
-	MSVideoSize vsize;
-	bool_t capture;
-	bool_t show_local;
-	bool_t display;
-	bool_t selfview; /*during calls*/
-}video_config_t;
-
-typedef struct ui_config
-{
-	int is_daemon;
-	int is_applet;
-	unsigned int timer_id;  /* the timer id for registration */
-}ui_config_t;
-
-
-
-typedef struct autoreplier_config
-{
-	int enabled;
-	int after_seconds;		/* accept the call after x seconds*/
-	int max_users;			/* maximum number of user that can call simultaneously */
-	int max_rec_time;  	/* the max time of incoming voice recorded */
-	int max_rec_msg;		/* maximum number of recorded messages */
-	const char *message;		/* the path of the file to be played */
-}autoreplier_config_t;
-
-
-struct _LinphoneCore;
-struct _sdp_context;
 struct _SipSetupContext;
+
+/**
+ * The LinphoneCall object represents a call issued or received by the LinphoneCore
+**/
 struct _LinphoneCall;
+typedef struct _LinphoneCall LinphoneCall;
 
+bool_t linphone_call_asked_to_autoanswer(struct _LinphoneCall *call);
+	
+/**
+ * Enum representing the direction of a call.
+ * @ingroup call_logs
+**/
+enum _LinphoneCallDir {
+	LinphoneCallOutgoing, /**< outgoing calls*/
+	LinphoneCallIncoming  /**< incoming calls*/
+};
 
-typedef enum _LinphoneCallDir {LinphoneCallOutgoing, LinphoneCallIncoming} LinphoneCallDir;
-
+/**
+ * Typedef for enum
+ * @ingroup call_logs
+**/
+typedef enum _LinphoneCallDir LinphoneCallDir;
 
+/**
+ * Enum representing the status of a call
+ * @ingroup call_logs
+**/
 typedef enum _LinphoneCallStatus { 
-	LinphoneCallSuccess,
-	LinphoneCallAborted,
-	LinphoneCallMissed
+	LinphoneCallSuccess, /**< The call was sucessful*/
+	LinphoneCallAborted, /**< The call was aborted */
+	LinphoneCallMissed /**< The call was missed (unanswered)*/
 } LinphoneCallStatus;
 
+/**
+ * Structure representing a call log.
+ *
+ * @ingroup call_logs
+ * 
+**/
 typedef struct _LinphoneCallLog{
-	LinphoneCallDir dir;
-	LinphoneCallStatus status;
-	char *from;
-	char *to;
-	char start_date[128];
-	int duration;
-	
+	LinphoneCallDir dir; /**< The direction of the call*/
+	LinphoneCallStatus status; /**< The status of the call*/
+	LinphoneAddress *from; /**<Originator of the call as a LinphoneAddress object*/
+	LinphoneAddress *to; /**<Destination of the call as a LinphoneAddress object*/
+	char start_date[128]; /**<Human readable string containg the start date*/
+	int duration; /**<Duration of the call in seconds*/
+	char *refkey;
+	void *user_pointer;
+	struct _LinphoneCore *lc;
 } LinphoneCallLog;
 
 
 
 /*public: */
+void linphone_call_log_set_user_pointer(LinphoneCallLog *cl, void *up);
+void *linphone_call_log_get_user_pointer(const LinphoneCallLog *cl);
+void linphone_call_log_set_ref_key(LinphoneCallLog *cl, const char *refkey);
+const char *linphone_call_log_get_ref_key(const LinphoneCallLog *cl);
 char * linphone_call_log_to_str(LinphoneCallLog *cl);
 
 typedef enum{
@@ -184,7 +146,7 @@ typedef enum{
 }LinphoneSubscribePolicy;
 
 typedef enum _LinphoneOnlineStatus{
-	LINPHONE_STATUS_UNKNOWN,
+	LINPHONE_STATUS_OFFLINE,
 	LINPHONE_STATUS_ONLINE,
 	LINPHONE_STATUS_BUSY,
 	LINPHONE_STATUS_BERIGHTBACK,
@@ -194,27 +156,15 @@ typedef enum _LinphoneOnlineStatus{
 	LINPHONE_STATUS_NOT_DISTURB,
 	LINPHONE_STATUS_MOVED,
 	LINPHONE_STATUS_ALT_SERVICE,
-	LINPHONE_STATUS_OFFLINE,
 	LINPHONE_STATUS_PENDING,
-	LINPHONE_STATUS_CLOSED,
 	LINPHONE_STATUS_END
 }LinphoneOnlineStatus;
 
 const char *linphone_online_status_to_string(LinphoneOnlineStatus ss);
 
-typedef struct _LinphoneFriend{
-	osip_from_t *url;
-	int in_did;
-	int out_did;
-	int sid;
-	int nid;
-	LinphoneSubscribePolicy pol;
-	LinphoneOnlineStatus status;
-	struct _LinphoneProxyConfig *proxy;
-	struct _LinphoneCore *lc;
-	bool_t subscribe;
-	bool_t inc_subscribe_pending;
-}LinphoneFriend;	
+struct _LinphoneFriend;
+
+typedef struct _LinphoneFriend LinphoneFriend;
 
 LinphoneFriend * linphone_friend_new();
 LinphoneFriend *linphone_friend_new_with_addr(const char *addr);
@@ -222,72 +172,91 @@ int linphone_friend_set_sip_addr(LinphoneFriend *fr, const char *uri);
 int linphone_friend_set_name(LinphoneFriend *fr, const char *name);
 int linphone_friend_send_subscribe(LinphoneFriend *fr, bool_t val);
 int linphone_friend_set_inc_subscribe_policy(LinphoneFriend *fr, LinphoneSubscribePolicy pol);
-int linphone_friend_set_proxy(LinphoneFriend *fr, struct _LinphoneProxyConfig *cfg);
 void linphone_friend_edit(LinphoneFriend *fr);
 void linphone_friend_done(LinphoneFriend *fr);
 void linphone_friend_destroy(LinphoneFriend *lf);
-/* memory returned by those 3 functions must be freed */
-char *linphone_friend_get_name(LinphoneFriend *lf);
-char *linphone_friend_get_addr(LinphoneFriend *lf);
-char *linphone_friend_get_url(LinphoneFriend *lf);	/* name <sip address> */
+const LinphoneAddress *linphone_friend_get_address(const LinphoneFriend *lf);
 bool_t linphone_friend_get_send_subscribe(const LinphoneFriend *lf);
 LinphoneSubscribePolicy linphone_friend_get_inc_subscribe_policy(const LinphoneFriend *lf);
 LinphoneOnlineStatus linphone_friend_get_status(const LinphoneFriend *lf);
-#define linphone_friend_in_list(lf)	((lf)->lc!=NULL)
+BuddyInfo * linphone_friend_get_info(const LinphoneFriend *lf);
+void linphone_friend_set_ref_key(LinphoneFriend *lf, const char *key);
+const char *linphone_friend_get_ref_key(const LinphoneFriend *lf);
+bool_t linphone_friend_in_list(const LinphoneFriend *lf);
 
 #define linphone_friend_url(lf) ((lf)->url)
 
-void linphone_friend_write_to_config_file(struct _LpConfig *config, LinphoneFriend *lf, int index);
-LinphoneFriend * linphone_friend_new_from_config_file(struct _LinphoneCore *lc, int index);
 
-typedef struct _LinphoneProxyConfig
-{
-	struct _LinphoneCore *lc;
-	char *reg_proxy;
-	char *reg_identity;
-	char *reg_route;
-	char *realm;
-	int expires;
-	int reg_time;
-	int rid;
-	char *type;
-	struct _SipSetupContext *ssctx;
-	int auth_failures;
-	char *contact_addr; /* our IP address as seen by the proxy, read from via 's received= parameter*/
-	int contact_port; /*our IP port as seen by the proxy, read from via's rport= parameter */
-	bool_t commit;
-	bool_t reg_sendregister;
-	bool_t registered;
-	bool_t publish;
-} LinphoneProxyConfig;
+/**
+ * @addtogroup proxies
+ * @{
+**/
+/**
+ * The LinphoneProxyConfig object represents a proxy configuration to be used
+ * by the LinphoneCore object.
+ * Its fields must not be used directly in favour of the accessors methods.
+ * Once created and filled properly the LinphoneProxyConfig can be given to
+ * LinphoneCore with linphone_core_add_proxy_config().
+ * This will automatically triggers the registration, if enabled.
+ *
+ * The proxy configuration are persistent to restarts because they are saved
+ * in the configuration file. As a consequence, after linphone_core_new() there
+ * might already be a list of configured proxy that can be examined with
+ * linphone_core_get_proxy_config_list().
+ *
+ * The default proxy (see linphone_core_set_default_proxy() ) is the one of the list
+ * that is used by default for calls.
+**/
+typedef struct _LinphoneProxyConfig LinphoneProxyConfig;
 
 LinphoneProxyConfig *linphone_proxy_config_new(void);
 int linphone_proxy_config_set_server_addr(LinphoneProxyConfig *obj, const char *server_addr);
-void linphone_proxy_config_set_identity(LinphoneProxyConfig *obj, const char *identity);
-void linphone_proxy_config_set_route(LinphoneProxyConfig *obj, const char *route);
-void linphone_proxy_config_expires(LinphoneProxyConfig *obj, const int expires);
+int linphone_proxy_config_set_identity(LinphoneProxyConfig *obj, const char *identity);
+int linphone_proxy_config_set_route(LinphoneProxyConfig *obj, const char *route);
+void linphone_proxy_config_expires(LinphoneProxyConfig *obj, int expires);
 void linphone_proxy_config_enable_register(LinphoneProxyConfig *obj, bool_t val);
 #define linphone_proxy_config_enableregister linphone_proxy_config_enable_register
 void linphone_proxy_config_edit(LinphoneProxyConfig *obj);
 int linphone_proxy_config_done(LinphoneProxyConfig *obj);
 void linphone_proxy_config_enable_publish(LinphoneProxyConfig *obj, bool_t val);
+void linphone_proxy_config_set_dial_escape_plus(LinphoneProxyConfig *cfg, bool_t val);
+void linphone_proxy_config_set_dial_prefix(LinphoneProxyConfig *cfg, const char *prefix);
+
 bool_t linphone_proxy_config_is_registered(const LinphoneProxyConfig *obj);
 const char *linphone_proxy_config_get_domain(const LinphoneProxyConfig *cfg);
 
-#define linphone_proxy_config_get_route(obj)  ((obj)->reg_route)
-#define linphone_proxy_config_get_identity(obj)	((obj)->reg_identity)
-#define linphone_proxy_config_publish_enabled(obj) ((obj)->publish)
-#define linphone_proxy_config_get_addr(obj) ((obj)->reg_proxy)
-#define linphone_proxy_config_get_expires(obj)	((obj)->expires)
-#define linphone_proxy_config_register_enabled(obj) ((obj)->reg_sendregister)
-#define linphone_proxy_config_get_core(obj) ((obj)->lc)
+const char *linphone_proxy_config_get_route(const LinphoneProxyConfig *obj);
+const char *linphone_proxy_config_get_identity(const LinphoneProxyConfig *obj);
+bool_t linphone_proxy_config_publish_enabled(const LinphoneProxyConfig *obj);
+const char *linphone_proxy_config_get_addr(const LinphoneProxyConfig *obj);
+int linphone_proxy_config_get_expires(const LinphoneProxyConfig *obj);
+bool_t linphone_proxy_config_register_enabled(const LinphoneProxyConfig *obj);
+struct _LinphoneCore * linphone_proxy_config_get_core(const LinphoneProxyConfig *obj);
+
+bool_t linphone_proxy_config_get_dial_escape_plus(const LinphoneProxyConfig *cfg);
+const char * linphone_proxy_config_get_dial_prefix(const LinphoneProxyConfig *cfg);
+
 /* destruction is called automatically when removing the proxy config */
 void linphone_proxy_config_destroy(LinphoneProxyConfig *cfg);
-LinphoneProxyConfig *linphone_proxy_config_new_from_config_file(struct _LpConfig *config, int index);
-void linphone_proxy_config_write_to_config_file(struct _LpConfig* config,LinphoneProxyConfig *obj, int index);
 void linphone_proxy_config_set_sip_setup(LinphoneProxyConfig *cfg, const char *type);
 SipSetupContext *linphone_proxy_config_get_sip_setup_context(LinphoneProxyConfig *cfg);
 SipSetup *linphone_proxy_config_get_sip_setup(LinphoneProxyConfig *cfg);
+/**
+ * normalize a human readable phone number into a basic string. 888-444-222 becomes 888444222
+ */
+int linphone_proxy_config_normalize_number(LinphoneProxyConfig *proxy, const char *username, char *result, size_t result_len);
+/*
+ *  attached a user data to a proxy config
+ */
+void linphone_proxy_config_set_user_data(LinphoneProxyConfig *cr, void * ud);
+/*
+ *  get user data to a proxy config. return null if any
+ */
+void * linphone_proxy_config_get_user_data(LinphoneProxyConfig *cr);
+
+/**
+ * @}
+**/
 
 typedef struct _LinphoneAccountCreator{
 	struct _LinphoneCore *lc;
@@ -308,34 +277,49 @@ int linphone_account_creator_test_existence(LinphoneAccountCreator *obj);
 LinphoneProxyConfig * linphone_account_creator_validate(LinphoneAccountCreator *obj);
 void linphone_account_creator_destroy(LinphoneAccountCreator *obj);
 
+struct _LinphoneAuthInfo;
 
-typedef struct _LinphoneAuthInfo
-{
-	char *username;
-	char *realm;
-	char *userid;
-	char *passwd;
-	char *ha1;
-	bool_t works;
-	bool_t first_time;
-}LinphoneAuthInfo;
+/**
+ * @ingroup authentication
+ * Object holding authentication information.
+ *
+ * @note The object's fields should not be accessed directly. Prefer using
+ * the accessor methods.
+ *
+ * In most case, authentication information consists of a username and password.
+ * Sometimes, a userid is required by proxy, and realm can be useful to discriminate
+ * different SIP domains.
+ *
+ * Once created and filled, a LinphoneAuthInfo must be added to the LinphoneCore in
+ * order to become known and used automatically when needed. 
+ * Use linphone_core_add_auth_info() for that purpose.
+ *
+ * The LinphoneCore object can take the initiative to request authentication information
+ * when needed to the application through the auth_info_requested callback of the
+ * LinphoneCoreVTable structure.
+ *
+ * The application can respond to this information request later using 
+ * linphone_core_add_auth_info(). This will unblock all pending authentication 
+ * transactions and retry them with authentication headers.
+ *
+**/
+typedef struct _LinphoneAuthInfo LinphoneAuthInfo;
 
 LinphoneAuthInfo *linphone_auth_info_new(const char *username, const char *userid,
 		const char *passwd, const char *ha1,const char *realm);
 void linphone_auth_info_set_passwd(LinphoneAuthInfo *info, const char *passwd);
 void linphone_auth_info_set_username(LinphoneAuthInfo *info, const char *username);
 void linphone_auth_info_set_userid(LinphoneAuthInfo *info, const char *userid);
+
+const char *linphone_auth_info_get_username(const LinphoneAuthInfo *i);
+const char *linphone_auth_info_get_passwd(const LinphoneAuthInfo *i);
+const char *linphone_auth_info_get_userid(const LinphoneAuthInfo *i);
+
 /* you don't need those function*/
 void linphone_auth_info_destroy(LinphoneAuthInfo *info);
 LinphoneAuthInfo * linphone_auth_info_new_from_config_file(struct _LpConfig *config, int pos);
 
-struct _LinphoneChatRoom{
-	struct _LinphoneCore *lc;
-	char  *peer;
-	char *route;
-	osip_from_t *peer_url;
-	void * user_data;
-};
+struct _LinphoneChatRoom;
 typedef struct _LinphoneChatRoom LinphoneChatRoom;
 
 LinphoneChatRoom * linphone_core_create_chat_room(struct _LinphoneCore *lc, const char *to);
@@ -385,47 +369,78 @@ void gstate_new_state(struct _LinphoneCore *lc, gstate_t new_state, const char *
 /*private*/
 void gstate_initialize(struct _LinphoneCore *lc) ;
 
-typedef void (*ShowInterfaceCb)(struct _LinphoneCore *lc);
-typedef void (*InviteReceivedCb)(struct _LinphoneCore *lc, const char *from);
-typedef void (*ByeReceivedCb)(struct _LinphoneCore *lc, const char *from);
-typedef void (*DisplayStatusCb)(struct _LinphoneCore *lc, const char *message);
-typedef void (*DisplayMessageCb)(struct _LinphoneCore *lc, const char *message);
-typedef void (*DisplayUrlCb)(struct _LinphoneCore *lc, const char *message, const char *url);
-typedef void (*DisplayQuestionCb)(struct _LinphoneCore *lc, const char *message);
-typedef void (*LinphoneCoreCbFunc)(struct _LinphoneCore *lc,void * user_data);
-typedef void (*NotifyReceivedCb)(struct _LinphoneCore *lc, LinphoneFriend * fid, const char *url, const char *status, const char *img);
-typedef void (*NewUnknownSubscriberCb)(struct _LinphoneCore *lc, LinphoneFriend *lf, const char *url);
-typedef void (*AuthInfoRequested)(struct _LinphoneCore *lc, const char *realm, const char *username);
-typedef void (*CallLogUpdated)(struct _LinphoneCore *lc, struct _LinphoneCallLog *newcl);
-typedef void (*TextMessageReceived)(struct _LinphoneCore *lc, LinphoneChatRoom *room, const char *from, const char *message);
-typedef void (*GeneralStateChange)(struct _LinphoneCore *lc, LinphoneGeneralState *gstate);
-typedef void (*DtmfReceived)(struct _LinphoneCore* lc, int dtmf);
-typedef void (*ReferReceived)(struct _LinphoneCore *lc, const char *refer_to);
+/**
+ * @addtogroup initializing
+ * @{
+**/
 
+/** Callback prototype */
+typedef void (*ShowInterfaceCb)(struct _LinphoneCore *lc);
+/** Callback prototype */
+typedef void (*InviteReceivedCb)(struct _LinphoneCore *lc, const char *from);
+/** Callback prototype */
+typedef void (*ByeReceivedCb)(struct _LinphoneCore *lc, const char *from);
+/** Callback prototype */
+typedef void (*DisplayStatusCb)(struct _LinphoneCore *lc, const char *message);
+/** Callback prototype */
+typedef void (*DisplayMessageCb)(struct _LinphoneCore *lc, const char *message);
+/** Callback prototype */
+typedef void (*DisplayUrlCb)(struct _LinphoneCore *lc, const char *message, const char *url);
+/** Callback prototype */
+typedef void (*DisplayQuestionCb)(struct _LinphoneCore *lc, const char *message);
+/** Callback prototype */
+typedef void (*LinphoneCoreCbFunc)(struct _LinphoneCore *lc,void * user_data);
+/** Callback prototype */
+typedef void (*NotifyReceivedCb)(struct _LinphoneCore *lc, const char *from, const char *msg);
+/** Callback prototype */
+typedef void (*NotifyPresenceReceivedCb)(struct _LinphoneCore *lc, LinphoneFriend * fid);
+/** Callback prototype */
+typedef void (*NewUnknownSubscriberCb)(struct _LinphoneCore *lc, LinphoneFriend *lf, const char *url);
+/** Callback prototype */
+typedef void (*AuthInfoRequested)(struct _LinphoneCore *lc, const char *realm, const char *username);
+/** Callback prototype */
+typedef void (*CallLogUpdated)(struct _LinphoneCore *lc, struct _LinphoneCallLog *newcl);
+/** Callback prototype */
+typedef void (*TextMessageReceived)(struct _LinphoneCore *lc, LinphoneChatRoom *room, const char *from, const char *message);
+/** Callback prototype */
+typedef void (*GeneralStateChange)(struct _LinphoneCore *lc, LinphoneGeneralState *gstate);
+/** Callback prototype */
+typedef void (*DtmfReceived)(struct _LinphoneCore* lc, int dtmf);
+/** Callback prototype */
+typedef void (*ReferReceived)(struct _LinphoneCore *lc, const char *refer_to);
+/** Callback prototype */
+typedef void (*BuddyInfoUpdated)(struct _LinphoneCore *lc, LinphoneFriend *lf);
+
+/**
+ * This structure holds all callbacks that the application should implement.
+ * 
+**/
 typedef struct _LinphoneVTable
 {
-	ShowInterfaceCb show;
-	InviteReceivedCb inv_recv;
-	ByeReceivedCb bye_recv;
-	NotifyReceivedCb notify_recv;
-	NewUnknownSubscriberCb new_unknown_subscriber;
-	AuthInfoRequested auth_info_requested;
-	DisplayStatusCb display_status;
-	DisplayMessageCb display_message;
-#ifdef VINCENT_MAURY_RSVP
-	/* the yes/no dialog box */
-	DisplayMessageCb display_yes_no;
-#endif
-	DisplayMessageCb display_warning;
+	ShowInterfaceCb show; /**< Notifies the application that it should show up*/
+	InviteReceivedCb inv_recv; /**< Notifies incoming calls */
+	ByeReceivedCb bye_recv; /**< Notify calls terminated by far end*/
+	NotifyPresenceReceivedCb notify_presence_recv; /**< Notify received presence events*/
+	NewUnknownSubscriberCb new_unknown_subscriber; /**< Notify about unknown subscriber */
+	AuthInfoRequested auth_info_requested; /**< Ask the application some authentication information */
+	DisplayStatusCb display_status; /**< Callback that notifies various events with human readable text.*/
+	DisplayMessageCb display_message;/**< Callback to display a message to the user */
+	DisplayMessageCb display_warning;/** Callback to display a warning to the user */
 	DisplayUrlCb display_url;
 	DisplayQuestionCb display_question;
-	CallLogUpdated call_log_updated;
-	TextMessageReceived text_received;
-	GeneralStateChange general_state;
-	DtmfReceived dtmf_received;
-	ReferReceived refer_received;
+	CallLogUpdated call_log_updated; /**< Notifies that call log list has been updated */
+	TextMessageReceived text_received; /**< A text message has been received */
+	GeneralStateChange general_state; /**< State notification callback */
+	DtmfReceived dtmf_received; /**< A dtmf has been received received */
+	ReferReceived refer_received; /**< A refer was received */
+	BuddyInfoUpdated buddy_info_updated; /**< a LinphoneFriend's BuddyInfo has changed*/
+	NotifyReceivedCb notify_recv; /**< Other notifications*/
 } LinphoneCoreVTable;
 
+/**
+ * @}
+**/
+
 typedef struct _LCCallbackObj
 {
   LinphoneCoreCbFunc _func;
@@ -447,68 +462,7 @@ typedef enum _LinphoneWaitingState{
 } LinphoneWaitingState;
 typedef void * (*LinphoneWaitingCallback)(struct _LinphoneCore *lc, void *context, LinphoneWaitingState ws, const char *purpose, float progress);
 
-
-typedef struct _LinphoneCore
-{
-	LinphoneCoreVTable vtable;
-	struct _LpConfig *config;
-	net_config_t net_conf;
-	sip_config_t sip_conf;
-	rtp_config_t rtp_conf;
-	sound_config_t sound_conf;
-	video_config_t video_conf;
-	codecs_config_t codecs_conf;
-	ui_config_t ui_conf;
-	autoreplier_config_t autoreplier_conf;
-	LinphoneProxyConfig *default_proxy;
-	MSList *friends;
-	MSList *auth_info;
-	struct _RingStream *ringstream;
-	LCCallbackObj preview_finished_cb;
-	bool_t preview_finished;
-	struct _LinphoneCall *call;   /* the current call, in the future it will be a list of calls (conferencing)*/
-	int rid; /*registration id*/
-	MSList *queued_calls;	/* used by the autoreplier */
-	MSList *call_logs;
-	MSList *chatrooms;
-	int max_call_logs;
-	int missed_calls;
-	struct _AudioStream *audiostream;  /**/
-	struct _VideoStream *videostream;
-	struct _VideoStream *previewstream;
-	struct _RtpProfile *local_profile;
-	MSList *subscribers;	/* unknown subscribers */
-	int minutes_away;
-	LinphoneOnlineStatus presence_mode;
-	LinphoneOnlineStatus prev_mode;
-	char *alt_contact;
-	void *data;
-	ms_mutex_t lock;
-	char *play_file;
-	char *rec_file;
-	time_t prevtime;
-	int dw_audio_bw;
-	int up_audio_bw;
-	int dw_video_bw;
-	int up_video_bw;
-	int audio_bw;
-	int automatic_action;
-	gstate_t gstate_power;
-	gstate_t gstate_reg;
-	gstate_t gstate_call;
-	LinphoneWaitingCallback wait_cb;
-	void *wait_ctx;
-	bool_t use_files;
-	bool_t apply_nat_settings;
-	bool_t ready;
-#ifdef VINCENT_MAURY_RSVP
-	/* QoS parameters*/
-	int rsvp_enable;
-	int rpc_enable;
-#endif
-} LinphoneCore;
-
-
+typedef struct _LinphoneCore LinphoneCore;
 
 /* THE main API */
 
@@ -520,22 +474,25 @@ void linphone_core_set_user_agent(const char *ua_name, const char *version);
 const char *linphone_core_get_version(void);
 
 LinphoneCore *linphone_core_new(const LinphoneCoreVTable *vtable,
-						const char *config_path, void* userdata);
-
-void linphone_core_init(LinphoneCore *lc, const LinphoneCoreVTable *vtable,
-						const char *config_path, void * userdata);
+						const char *config_path, const char *factory_config, void* userdata);
 
 /* function to be periodically called in a main loop */
 void linphone_core_iterate(LinphoneCore *lc);
 
+LinphoneAddress * linphone_core_interpret_url(LinphoneCore *lc, const char *url);
+
 int linphone_core_invite(LinphoneCore *lc, const char *url);
 
+int linphone_core_invite_address(LinphoneCore *lc, const LinphoneAddress *addr);
+
 int linphone_core_refer(LinphoneCore *lc, const char *url);
 
 bool_t linphone_core_inc_invite_pending(LinphoneCore*lc);
 
 bool_t linphone_core_in_call(const LinphoneCore *lc);
 
+LinphoneCall *linphone_core_get_current_call(LinphoneCore *lc);
+
 int linphone_core_accept_call(LinphoneCore *lc, const char *url);
 
 int linphone_core_terminate_call(LinphoneCore *lc, const char *url);
@@ -552,7 +509,7 @@ bool_t linphone_core_get_guess_hostname(LinphoneCore *lc);
 bool_t linphone_core_ipv6_enabled(LinphoneCore *lc);
 void linphone_core_enable_ipv6(LinphoneCore *lc, bool_t val);
 
-osip_from_t *linphone_core_get_primary_contact_parsed(LinphoneCore *lc);
+LinphoneAddress *linphone_core_get_primary_contact_parsed(LinphoneCore *lc);
 
 /*0= no bandwidth limit*/
 void linphone_core_set_download_bandwidth(LinphoneCore *lc, int bw);
@@ -560,7 +517,17 @@ void linphone_core_set_upload_bandwidth(LinphoneCore *lc, int bw);
 
 int linphone_core_get_download_bandwidth(const LinphoneCore *lc);
 int linphone_core_get_upload_bandwidth(const LinphoneCore *lc);
-
+/**
+ * set audio packetization time linphone expect to received from peer
+ * @ingroup media_parameters
+ *
+ */
+void linphone_core_set_download_ptime(LinphoneCore *lc, int ptime);
+/**
+ * get audio packetization time linphone expect to received from peer, 0 means unspecified
+ * @ingroup media_parameters
+ */
+int  linphone_core_get_download_ptime(LinphoneCore *lc);
 
 #ifdef VINCENT_MAURY_RSVP
 /* QoS functions */
@@ -578,10 +545,18 @@ const MSList *linphone_core_get_video_codecs(const LinphoneCore *lc);
 
 int linphone_core_set_video_codecs(LinphoneCore *lc, MSList *codecs);
 
+bool_t linphone_core_payload_type_enabled(LinphoneCore *lc, PayloadType *pt);
+
+int linphone_core_enable_payload_type(LinphoneCore *lc, PayloadType *pt, bool_t enable);
+
+const char *linphone_core_get_payload_type_description(LinphoneCore *lc, PayloadType *pt);
+
 bool_t linphone_core_check_payload_type_usability(LinphoneCore *lc, PayloadType *pt);
 
 int linphone_core_add_proxy_config(LinphoneCore *lc, LinphoneProxyConfig *config);
 
+void linphone_core_clear_proxy_config(LinphoneCore *lc);
+
 void linphone_core_remove_proxy_config(LinphoneCore *lc, LinphoneProxyConfig *config);
 
 const MSList *linphone_core_get_proxy_config_list(const LinphoneCore *lc);
@@ -592,13 +567,13 @@ void linphone_core_set_default_proxy_index(LinphoneCore *lc, int index);
 
 int linphone_core_get_default_proxy(LinphoneCore *lc, LinphoneProxyConfig **config);
 
-void linphone_core_add_auth_info(LinphoneCore *lc, LinphoneAuthInfo *info);
+void linphone_core_add_auth_info(LinphoneCore *lc, const LinphoneAuthInfo *info);
 
-void linphone_core_remove_auth_info(LinphoneCore *lc, LinphoneAuthInfo *info);
+void linphone_core_remove_auth_info(LinphoneCore *lc, const LinphoneAuthInfo *info);
 
 const MSList *linphone_core_get_auth_info_list(const LinphoneCore *lc);
 
-LinphoneAuthInfo *linphone_core_find_auth_info(LinphoneCore *lc, const char *realm, const char *username);
+const LinphoneAuthInfo *linphone_core_find_auth_info(LinphoneCore *lc, const char *realm, const char *username);
 
 void linphone_core_abort_authentication(LinphoneCore *lc,  LinphoneAuthInfo *info);
 
@@ -624,10 +599,16 @@ void linphone_core_set_use_info_for_dtmf(LinphoneCore *lc, bool_t use_info);
 
 bool_t linphone_core_get_use_info_for_dtmf(LinphoneCore *lc);
 
+void linphone_core_set_use_rfc2833_for_dtmf(LinphoneCore *lc,bool_t use_rfc2833);
+
+bool_t linphone_core_get_use_rfc2833_for_dtmf(LinphoneCore *lc);
+
 int linphone_core_get_sip_port(LinphoneCore *lc);
 
 void linphone_core_set_sip_port(LinphoneCore *lc,int port);
 
+ortp_socket_t linphone_core_get_sip_socket(LinphoneCore *lc);
+
 void linphone_core_set_inc_timeout(LinphoneCore *lc, int seconds);
 
 int linphone_core_get_inc_timeout(LinphoneCore *lc);
@@ -658,6 +639,18 @@ int linphone_core_get_play_level(LinphoneCore *lc);
 int linphone_core_get_rec_level(LinphoneCore *lc);
 void linphone_core_set_ring_level(LinphoneCore *lc, int level);
 void linphone_core_set_play_level(LinphoneCore *lc, int level);
+/**
+ * Allow to control play level before entering  sound card:  level in db
+ *
+ * @ingroup media_parameters
+**/
+void linphone_core_set_soft_play_level(LinphoneCore *lc, float level);
+/**
+ * get play level before entering  sound card:  level in db
+ *
+ * @ingroup media_parameters
+**/
+float linphone_core_get_soft_play_level(LinphoneCore *lc);
 void linphone_core_set_rec_level(LinphoneCore *lc, int level);
 const char * linphone_core_get_ringer_device(LinphoneCore *lc);
 const char * linphone_core_get_playback_device(LinphoneCore *lc);
@@ -672,8 +665,8 @@ const char *linphone_core_get_ring(const LinphoneCore *lc);
 void linphone_core_set_ringback(LinphoneCore *lc, const char *path);
 const char * linphone_core_get_ringback(const LinphoneCore *lc);
 int linphone_core_preview_ring(LinphoneCore *lc, const char *ring,LinphoneCoreCbFunc func,void * userdata);
-void linphone_core_enable_echo_cancelation(LinphoneCore *lc, bool_t val);
-bool_t linphone_core_echo_cancelation_enabled(LinphoneCore *lc);
+void linphone_core_enable_echo_cancellation(LinphoneCore *lc, bool_t val);
+bool_t linphone_core_echo_cancellation_enabled(LinphoneCore *lc);
 
 void linphone_core_enable_echo_limiter(LinphoneCore *lc, bool_t val);
 bool_t linphone_core_echo_limiter_enabled(const LinphoneCore *lc);
@@ -682,6 +675,12 @@ void linphone_core_enable_agc(LinphoneCore *lc, bool_t val);
 bool_t linphone_core_agc_enabled(const LinphoneCore *lc);
 
 void linphone_core_mute_mic(LinphoneCore *lc, bool_t muted);
+/**
+ * return mic state.
+ *
+ * @ingroup media_parameters
+**/
+bool_t linphone_core_is_mic_muted(LinphoneCore *lc);
 
 void linphone_core_set_presence_info(LinphoneCore *lc,int minutes_away,const char *contact,LinphoneOnlineStatus os);
 
@@ -692,12 +691,15 @@ void linphone_core_add_friend(LinphoneCore *lc, LinphoneFriend *fr);
 void linphone_core_remove_friend(LinphoneCore *lc, LinphoneFriend *fr);
 void linphone_core_reject_subscriber(LinphoneCore *lc, LinphoneFriend *lf);
 /* a list of LinphoneFriend */
-const MSList * linphone_core_get_friend_list(LinphoneCore *lc);
+const MSList * linphone_core_get_friend_list(const LinphoneCore *lc);
 /* notify all friends that have subscribed */
 void linphone_core_notify_all_friends(LinphoneCore *lc, LinphoneOnlineStatus os);
+LinphoneFriend *linphone_core_get_friend_by_address(const LinphoneCore *lc, const char *addr);
+LinphoneFriend *linphone_core_get_friend_by_ref_key(const LinphoneCore *lc, const char *key);
 
 /* returns a list of LinphoneCallLog */
-MSList * linphone_core_get_call_logs(LinphoneCore *lc);
+const MSList * linphone_core_get_call_logs(LinphoneCore *lc);
+void linphone_core_clear_call_logs(LinphoneCore *lc);
 
 /* video support */
 void linphone_core_enable_video(LinphoneCore *lc, bool_t vcap_enabled, bool_t display_enabled);
@@ -725,6 +727,9 @@ const char**  linphone_core_get_video_devices(const LinphoneCore *lc);
 int linphone_core_set_video_device(LinphoneCore *lc, const char *id);
 const char *linphone_core_get_video_device(const LinphoneCore *lc);
 
+/* Set static picture to be used when "Static picture" is the video device */
+int linphone_core_set_static_picture(LinphoneCore *lc, const char *path);
+
 /*function to be used for eventually setting window decorations (icons, title...)*/
 unsigned long linphone_core_get_native_video_window_id(const LinphoneCore *lc);
 
@@ -736,12 +741,24 @@ void linphone_core_set_record_file(LinphoneCore *lc, const char *file);
 
 gstate_t linphone_core_get_state(const LinphoneCore *lc, gstate_group_t group);
 int linphone_core_get_current_call_duration(const LinphoneCore *lc);
-const char *linphone_core_get_remote_uri(LinphoneCore *lc);
+const LinphoneAddress *linphone_core_get_remote_uri(LinphoneCore *lc);
 
 int linphone_core_get_mtu(const LinphoneCore *lc);
 void linphone_core_set_mtu(LinphoneCore *lc, int mtu);
 
-bool_t linphone_core_is_in_main_thread(LinphoneCore *lc);
+/**
+ * This method is called by the application to notify the linphone core library when network is reachable.
+ * Calling this method with true trigger linphone to initiate a registration process for all proxy
+ * configuration with parameter register set to enable.
+ * This method disable the automatic registration mode. It means you must call this method after each network state changes
+ *
+ */
+void linphone_core_set_network_reachable(LinphoneCore* lc,bool_t value);
+/**
+ * return network state either as positioned by the application or by linphone
+ */
+bool_t linphone_core_is_network_reachabled(LinphoneCore* lc);
+
 
 void *linphone_core_get_user_data(LinphoneCore *lc);
 
@@ -763,23 +780,10 @@ void linphone_core_set_waiting_callback(LinphoneCore *lc, LinphoneWaitingCallbac
 /*returns the list of registered SipSetup (linphonecore plugins) */
 const MSList * linphone_core_get_sip_setups(LinphoneCore *lc);
 
-void linphone_core_uninit(LinphoneCore *lc);
 void linphone_core_destroy(LinphoneCore *lc);
 
-/* end of lecacy api */
-
-/*internal use only */
-#define linphone_core_lock(lc)	ms_mutex_lock(&(lc)->lock)
-#define linphone_core_unlock(lc)	ms_mutex_unlock((&lc)->lock)
-void linphone_core_start_media_streams(LinphoneCore *lc, struct _LinphoneCall *call);
-void linphone_core_stop_media_streams(LinphoneCore *lc);
-const char * linphone_core_get_identity(LinphoneCore *lc);
-const char * linphone_core_get_route(LinphoneCore *lc);
-bool_t linphone_core_interpret_url(LinphoneCore *lc, const char *url, char **real_url, osip_to_t **real_parsed_url, char **route);
-void linphone_core_start_waiting(LinphoneCore *lc, const char *purpose);
-void linphone_core_update_progress(LinphoneCore *lc, const char *purpose, float progresses);
-void linphone_core_stop_waiting(LinphoneCore *lc);
-
+/*for advanced users:*/
+void linphone_core_set_audio_transports(LinphoneCore *lc, RtpTransport *rtp, RtpTransport *rtcp);
 
 #ifdef __cplusplus
 }
