@@ -20,11 +20,13 @@ Copyright (C) 2000  Simon MORLAT (simon.morlat@linphone.org)
  
 #include "linphonecore.h"
 #include "sipsetup.h"
-#include <eXosip2/eXosip.h>
-#include <osipparser2/osip_message.h>
 #include "lpconfig.h"
 #include "private.h"
 
+
+#include <ctype.h>
+
+
 void linphone_proxy_config_write_all_to_config_file(LinphoneCore *lc){
 	MSList *elem;
 	int i;
@@ -36,10 +38,17 @@ void linphone_proxy_config_write_all_to_config_file(LinphoneCore *lc){
 
 void linphone_proxy_config_init(LinphoneProxyConfig *obj){
 	memset(obj,0,sizeof(LinphoneProxyConfig));
-	obj->rid=-1;
 	obj->expires=3600;
 }
 
+/**
+ * @addtogroup proxies
+ * @{
+**/
+
+/**
+ * Creates an empty proxy config.
+**/
 LinphoneProxyConfig *linphone_proxy_config_new(){
 	LinphoneProxyConfig *obj=NULL;
 	obj=ms_new(LinphoneProxyConfig,1);
@@ -47,6 +56,12 @@ LinphoneProxyConfig *linphone_proxy_config_new(){
 	return obj;
 }
 
+/**
+ * Destroys a proxy config.
+ * 
+ * @note: LinphoneProxyConfig that have been removed from LinphoneCore with
+ * linphone_core_remove_proxy_config() must not be freed.
+**/
 void linphone_proxy_config_destroy(LinphoneProxyConfig *obj){
 	if (obj->reg_proxy!=NULL) ms_free(obj->reg_proxy);
 	if (obj->reg_identity!=NULL) ms_free(obj->reg_identity);
@@ -54,165 +69,105 @@ void linphone_proxy_config_destroy(LinphoneProxyConfig *obj){
 	if (obj->ssctx!=NULL) sip_setup_context_free(obj->ssctx);
 	if (obj->realm!=NULL) ms_free(obj->realm);
 	if (obj->type!=NULL) ms_free(obj->type);
-	if (obj->contact_addr!=NULL) ms_free(obj->contact_addr);
+	if (obj->dial_prefix!=NULL) ms_free(obj->dial_prefix);
+	if (obj->op) sal_op_release(obj->op);
 }
 
+/**
+ * Returns a boolean indicating that the user is sucessfully registered on the proxy.
+**/
 bool_t linphone_proxy_config_is_registered(const LinphoneProxyConfig *obj){
 	return obj->registered;
 }
 
-void linphone_proxy_config_get_contact(LinphoneProxyConfig *cfg, const char **ip, int *port){
-	if (cfg->registered){
-		*ip=cfg->contact_addr;
-		*port=cfg->contact_port;
-	}else{
-		*ip=NULL;
-		*port=0;
-	}
-}
-
-static void update_contact(LinphoneProxyConfig *cfg, const char *ip, const char *port){
-	if (cfg->contact_addr){
-		ms_free(cfg->contact_addr);
-	}
-	cfg->contact_addr=ms_strdup(ip);
-	if (port!=NULL)
-		cfg->contact_port=atoi(port);
-	else cfg->contact_port=5060;
-}
-
-bool_t linphone_proxy_config_register_again_with_updated_contact(LinphoneProxyConfig *obj, osip_message_t *orig_request, osip_message_t *last_answer){
-	osip_message_t *msg;
-	const char *rport,*received;
-	osip_via_t *via=NULL;
-	osip_generic_param_t *param=NULL;
-	osip_contact_t *ctt=NULL;
-	osip_message_get_via(last_answer,0,&via);
-	if (!via) return FALSE;
-	osip_via_param_get_byname(via,"rport",&param);
-	if (param) rport=param->gvalue;
-	else return FALSE;
-	param=NULL;
-	osip_via_param_get_byname(via,"received",&param);
-	if (param) received=param->gvalue;
-	else return FALSE;
-	osip_message_get_contact(orig_request,0,&ctt);
-	if (strcmp(ctt->url->host,received)==0){
-		/*ip address matches, check ports*/
-		const char *contact_port=ctt->url->port;
-		const char *via_rport=rport;
-		if (via_rport==NULL || strlen(via_rport)>0)
-			via_rport="5060";
-		if (contact_port==NULL || strlen(contact_port)>0)
-			contact_port="5060";
-		if (strcmp(contact_port,via_rport)==0){
-			ms_message("Register has up to date contact, doing nothing.");
-			return FALSE;
-		}else ms_message("ports do not match, need to update the register (%s <> %s)", contact_port,via_rport);
-	}
-	eXosip_lock();
-	msg=NULL;
-	eXosip_register_build_register(obj->rid,obj->expires,&msg);
-	if (msg==NULL){
-		eXosip_unlock();
-		ms_warning("Fail to create a contact updated register.");
-		return FALSE;
-	}
-	osip_message_get_contact(msg,0,&ctt);
-	if (ctt->url->host!=NULL){
-		osip_free(ctt->url->host);
-	}
-	ctt->url->host=osip_strdup(received);
-	if (ctt->url->port!=NULL){
-		osip_free(ctt->url->port);
-	}
-	ctt->url->port=osip_strdup(rport);
-	eXosip_register_send_register(obj->rid,msg);
-	eXosip_unlock();
-	update_contact(obj,received,rport);
-	ms_message("Resending new register with updated contact %s:%s",received,rport);
-	return TRUE;
-}
-
+/**
+ * Sets the proxy address
+ *
+ * Examples of valid sip proxy address are:
+ * - IP address: sip:87.98.157.38
+ * - IP address with port: sip:87.98.157.38:5062
+ * - hostnames : sip:sip.example.net
+**/
 int linphone_proxy_config_set_server_addr(LinphoneProxyConfig *obj, const char *server_addr){
-	int err;
-	osip_from_t *url;
+	LinphoneAddress *addr;
+	char *try=NULL;
+	
 	if (obj->reg_proxy!=NULL) ms_free(obj->reg_proxy);
 	obj->reg_proxy=NULL;
+	
 	if (server_addr!=NULL && strlen(server_addr)>0){
-		osip_from_init(&url);
-		err=osip_from_parse(url,server_addr);
-		if (err==0 && url->url->host!=NULL){
-			obj->reg_proxy=ms_strdup(server_addr);
+		addr=linphone_address_new(server_addr);
+		if (!addr){
+			/*try to prepend 'sip:' */
+			if (strstr(server_addr,"sip:")==NULL){
+				try=ms_strdup_printf("sip:%s",server_addr);
+				addr=linphone_address_new(try);
+				ms_free(try);
+			}
+		}
+		if (addr){
+			obj->reg_proxy=linphone_address_as_string_uri_only(addr);
+			linphone_address_destroy(addr);
 		}else{
 			ms_warning("Could not parse %s",server_addr);
+			return -1;
 		}
-		osip_from_free(url);
 	}
 	return 0;
 }
 
-void linphone_proxy_config_set_identity(LinphoneProxyConfig *obj, const char *identity){
-	int err=0;
-	osip_from_t *url=NULL;
+/**
+ * Sets the user identity as a SIP address.
+ *
+ * This identity is normally formed with display name, username and domain, such 
+ * as:
+ * Alice <sip:alice@example.net>
+ * The REGISTER messages will have from and to set to this identity.
+ *
+**/
+int linphone_proxy_config_set_identity(LinphoneProxyConfig *obj, const char *identity){
+	LinphoneAddress *addr;
 	if (identity!=NULL && strlen(identity)>0){
-		osip_from_init(&url);
-		err=osip_from_parse(url,identity);
-		if (err<0 || url->url->host==NULL || url->url->username==NULL){
-			ms_warning("Could not parse %s",identity);
-			osip_from_free(url);
-			return;
+		addr=linphone_address_new(identity);
+		if (!addr || linphone_address_get_username(addr)==NULL){
+			ms_warning("Invalid sip identity: %s",identity);
+			if (addr)
+				linphone_address_destroy(addr);
+			return -1;
+		}else{
+			if (obj->reg_identity!=NULL) {
+				ms_free(obj->reg_identity);
+				obj->reg_identity=NULL;
+			}
+			obj->reg_identity=ms_strdup(identity);
+			if (obj->realm){
+				ms_free(obj->realm);
+			}
+			obj->realm=ms_strdup(linphone_address_get_domain(addr));
+			linphone_address_destroy(addr);
+			return 0;
 		}
-	} else err=-2;
-	if (obj->reg_identity!=NULL) {
-		ms_free(obj->reg_identity);
-		obj->reg_identity=NULL;
 	}
-	if (err==-2) obj->reg_identity=NULL;
-	else {
-		obj->reg_identity=ms_strdup(identity);
-		if (obj->realm)
-			ms_free(obj->realm);
-		obj->realm=ms_strdup(url->url->host);
-	}
-	if (url) osip_from_free(url);
+	return -1;
 }
 
 const char *linphone_proxy_config_get_domain(const LinphoneProxyConfig *cfg){
 	return cfg->realm;
 }
 
-void linphone_proxy_config_set_route(LinphoneProxyConfig *obj, const char *route)
+/**
+ * Sets a SIP route.
+ * When a route is set, all outgoing calls will go to the route's destination if this proxy
+ * is the default one (see linphone_core_set_default_proxy() ).
+**/
+int linphone_proxy_config_set_route(LinphoneProxyConfig *obj, const char *route)
 {
-	int err;
-	osip_uri_param_t *lr_param=NULL;
-	osip_route_t *rt=NULL;
-	char *tmproute=NULL;
-	if (route!=NULL && strlen(route)>0){
-		osip_route_init(&rt);
-		err=osip_route_parse(rt,route);
-		if (err<0){
-			ms_warning("Could not parse %s",route);
-			osip_route_free(rt);
-			return ;
-		}
-		if (obj->reg_route!=NULL) {
-			ms_free(obj->reg_route);
-			obj->reg_route=NULL;
-		}
-			
-		/* check if the lr parameter is set , if not add it */
-		osip_uri_uparam_get_byname(rt->url, "lr", &lr_param);
-	  	if (lr_param==NULL){
-			osip_uri_uparam_add(rt->url,osip_strdup("lr"),NULL);
-			osip_route_to_str(rt,&tmproute);
-			obj->reg_route=ms_strdup(tmproute);
-			osip_free(tmproute);
-		}else obj->reg_route=ms_strdup(route);
-	}else{
-		if (obj->reg_route!=NULL) ms_free(obj->reg_route);
+	if (obj->reg_route!=NULL){
+		ms_free(obj->reg_route);
 		obj->reg_route=NULL;
 	}
+	obj->reg_route=ms_strdup(route);
+	return 0;
 }
 
 bool_t linphone_proxy_config_check(LinphoneCore *lc, LinphoneProxyConfig *obj){
@@ -231,10 +186,16 @@ bool_t linphone_proxy_config_check(LinphoneCore *lc, LinphoneProxyConfig *obj){
 	return TRUE;
 }
 
+/**
+ * Indicates whether a REGISTER request must be sent to the proxy.
+**/
 void linphone_proxy_config_enableregister(LinphoneProxyConfig *obj, bool_t val){
 	obj->reg_sendregister=val;
 }
 
+/**
+ * Sets the registration expiration time in seconds.
+**/
 void linphone_proxy_config_expires(LinphoneProxyConfig *obj, int val){
 	if (val<=0) val=600;
 	obj->expires=val;
@@ -244,16 +205,20 @@ void linphone_proxy_config_enable_publish(LinphoneProxyConfig *obj, bool_t val){
 	obj->publish=val;
 }
 
+/**
+ * Starts editing a proxy configuration.
+ *
+ * Because proxy configuration must be consistent, applications MUST
+ * call linphone_proxy_config_edit() before doing any attempts to modify
+ * proxy configuration (such as identity, proxy address and so on).
+ * Once the modifications are done, then the application must call
+ * linphone_proxy_config_done() to commit the changes.
+**/
 void linphone_proxy_config_edit(LinphoneProxyConfig *obj){
-	obj->auth_failures=0;
 	if (obj->reg_sendregister){
 		/* unregister */
 		if (obj->registered) {
-			osip_message_t *msg;
-			eXosip_lock();
-			eXosip_register_build_register(obj->rid,0,&msg);
-			eXosip_register_send_register(obj->rid,msg);
-			eXosip_unlock();
+			sal_unregister(obj->op);
 			obj->registered=FALSE;
 		}
 	}
@@ -270,16 +235,141 @@ static void linphone_proxy_config_register(LinphoneProxyConfig *obj){
 	if (obj->reg_identity!=NULL) id_str=obj->reg_identity;
 	else id_str=linphone_core_get_primary_contact(obj->lc);
 	if (obj->reg_sendregister){
-		char *ct=NULL;
-		osip_message_t *msg=NULL;
-		eXosip_lock();
-		obj->rid=eXosip_register_build_initial_register(id_str,obj->reg_proxy,NULL,obj->expires,&msg);
-		eXosip_register_send_register(obj->rid,msg);
-		eXosip_unlock();
-		if (ct!=NULL) osip_free(ct);
+		if (obj->op)
+			sal_op_release(obj->op);
+		obj->op=sal_op_new(obj->lc->sal);
+		sal_op_set_user_pointer(obj->op,obj);
+		sal_register(obj->op,obj->reg_proxy,obj->reg_identity,obj->expires);
 	}
 }
 
+
+/**
+ * Sets a dialing prefix to be automatically prepended when inviting a number with 
+ * #linphone_core_invite.
+ *
+**/
+void linphone_proxy_config_set_dial_prefix(LinphoneProxyConfig *cfg, const char *prefix){
+	if (cfg->dial_prefix!=NULL){
+		ms_free(cfg->dial_prefix);
+		cfg->dial_prefix=NULL;
+	}
+	if (prefix && prefix[0]!='\0') cfg->dial_prefix=ms_strdup(prefix);
+}
+
+/**
+ * Returns dialing prefix.
+ *
+ * 
+**/
+const char *linphone_proxy_config_get_dial_prefix(const LinphoneProxyConfig *cfg){
+	return cfg->dial_prefix;
+}
+
+/**
+ * Sets whether liblinphone should replace "+" by "00" in dialed numbers (passed to
+ * #linphone_core_invite ).
+ *
+**/
+void linphone_proxy_config_set_dial_escape_plus(LinphoneProxyConfig *cfg, bool_t val){
+	cfg->dial_escape_plus=val;
+}
+
+/**
+ * Returns whether liblinphone should replace "+" by "00" in dialed numbers (passed to
+ * #linphone_core_invite ).
+ *
+**/
+bool_t linphone_proxy_config_get_dial_escape_plus(const LinphoneProxyConfig *cfg){
+	return cfg->dial_escape_plus;
+}
+
+
+static bool_t is_a_phone_number(const char *username){
+	const char *p;
+	for(p=username;*p!='\0';++p){
+		if (isdigit(*p) || 
+		    *p==' ' ||
+		    *p=='-' ||
+		    *p==')' ||
+			*p=='(' ||
+			*p=='/' ||
+			*p=='+') continue;
+		else return FALSE;
+	}
+	return TRUE;
+}
+
+static char *flatten_number(const char *number){
+	char *result=ms_malloc0(strlen(number)+1);
+	char *w=result;
+	const char *r;
+	for(r=number;*r!='\0';++r){
+		if (*r=='+' || isdigit(*r)){
+			*w++=*r;
+		}
+	}
+	*w++='\0';
+	return result;
+}
+
+static void copy_result(const char *src, char *dest, size_t destlen, bool_t escape_plus){
+	int i=0;
+	
+	if (escape_plus && src[0]=='+' && destlen>2){
+		dest[0]='0';
+		dest[1]='0';
+		src++;
+		i=2;
+	}
+	
+	for(;(i<destlen-1) && *src!='\0';++i){
+		dest[i]=*src;
+		src++;
+	}
+	dest[i]='\0';
+}
+
+
+static char *append_prefix(const char *number, const char *prefix){
+	char *res=ms_malloc(strlen(number)+strlen(prefix)+1);
+	strcpy(res,prefix);
+	return strcat(res,number);
+}
+
+int linphone_proxy_config_normalize_number(LinphoneProxyConfig *proxy, const char *username, char *result, size_t result_len){
+	char *flatten;
+	int numlen;
+	if (is_a_phone_number(username)){
+		flatten=flatten_number(username);
+		ms_message("Flattened number is '%s'",flatten);
+		numlen=strlen(flatten);
+		if (numlen>10 || flatten[0]=='+' || proxy->dial_prefix==NULL || proxy->dial_prefix[0]=='\0'){
+			ms_message("No need to add a prefix");
+			/* prefix is already there */
+			copy_result(flatten,result,result_len,proxy->dial_escape_plus);
+			ms_free(flatten);
+			return 0;
+		}else if (proxy->dial_prefix && proxy->dial_prefix[0]!='\0'){
+			char *prefixed;
+			int skipped=0;
+			ms_message("Need to prefix with %s",proxy->dial_prefix);
+			if (numlen==10){
+				/*remove initial number before prepending prefix*/
+				skipped=1;
+			}
+			prefixed=append_prefix(flatten+skipped,proxy->dial_prefix);
+			ms_free(flatten);
+			copy_result(prefixed,result,result_len,proxy->dial_escape_plus);
+			ms_free(prefixed);
+		}
+	}else strncpy(result,username,result_len);
+	return 0;
+}
+
+/**
+ * Commits modification made to the proxy configuration.
+**/
 int linphone_proxy_config_done(LinphoneProxyConfig *obj)
 {
 	if (!linphone_proxy_config_check(obj->lc,obj)) return -1;
@@ -298,183 +388,85 @@ void linphone_proxy_config_set_realm(LinphoneProxyConfig *cfg, const char *realm
 }
 
 int linphone_proxy_config_send_publish(LinphoneProxyConfig *proxy,
-			       LinphoneOnlineStatus presence_mode)
-{
-  osip_message_t *pub;
-  int i;
-  const char *from=NULL;
-  char buf[5000];
-
-  if (proxy->publish==FALSE) return 0;
-	
-  if (proxy!=NULL) {
-    from=linphone_proxy_config_get_identity(proxy);
-  }
-  if (from==NULL) from=linphone_core_get_primary_contact(proxy->lc);
-
-  if (presence_mode==LINPHONE_STATUS_ONLINE)
-    {
-      snprintf(buf, 5000, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\
-<presence xmlns=\"urn:ietf:params:xml:ns:pidf\"\n\
-          entity=\"%s\">\n\
-<tuple id=\"sg89ae\">\n\
-<status>\n\
-<basic>open</basic>\n\
-</status>\n\
-<contact priority=\"0.8\">%s</contact>\n\
-<note>online</note>\n\
-</tuple>\n\
-</presence>",
-	       from, from);
-    }
-  else if (presence_mode==LINPHONE_STATUS_BUSY
-	   ||presence_mode==LINPHONE_STATUS_NOT_DISTURB)
-    {
-      snprintf(buf, 5000, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\
-<presence xmlns=\"urn:ietf:params:xml:ns:pidf\"\n\
-          xmlns:es=\"urn:ietf:params:xml:ns:pidf:status:rpid-status\"\n\
-          entity=\"%s\">\n\
-<tuple id=\"sg89ae\">\n\
-<status>\n\
-<basic>open</basic>\n\
-<es:activities>\n\
-  <es:activity>busy</es:activity>\n\
-</es:activities>\n\
-</status>\n\
-<contact priority=\"0.8\">%s</contact>\n\
-<note>busy</note>\n\
-</tuple>\n\
-</presence>",
-	      from, from);
-    }
-  else if (presence_mode==LINPHONE_STATUS_BERIGHTBACK)
-    {
-      snprintf(buf, 5000, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\
-<presence xmlns=\"urn:ietf:params:xml:ns:pidf\"\n\
-          xmlns:es=\"urn:ietf:params:xml:ns:pidf:status:rpid-status\"\n\
-          entity=\"%s\">\n\
-<tuple id=\"sg89ae\">\n\
-<status>\n\
-<basic>open</basic>\n\
-<es:activities>\n\
-  <es:activity>in-transit</es:activity>\n\
-</es:activities>\n\
-</status>\n\
-<contact priority=\"0.8\">%s</contact>\n\
-<note>be right back</note>\n\
-</tuple>\n\
-</presence>",
-	      from,from);
-    }
-  else if (presence_mode==LINPHONE_STATUS_AWAY
-	   ||presence_mode==LINPHONE_STATUS_MOVED
-	   ||presence_mode==LINPHONE_STATUS_ALT_SERVICE)
-    {
-      snprintf(buf, 5000, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\
-<presence xmlns=\"urn:ietf:params:xml:ns:pidf\"\n\
-          xmlns:es=\"urn:ietf:params:xml:ns:pidf:status:rpid-status\"\n\
-          entity=\"%s\">\n\
-<tuple id=\"sg89ae\">\n\
-<status>\n\
-<basic>open</basic>\n\
-<es:activities>\n\
-  <es:activity>away</es:activity>\n\
-</es:activities>\n\
-</status>\n\
-<contact priority=\"0.8\">%s</contact>\n\
-<note>away</note>\n\
-</tuple>\n\
-</presence>",
-	      from, from);
-    }
-  else if (presence_mode==LINPHONE_STATUS_ONTHEPHONE)
-    {
-      snprintf(buf, 5000, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\
-<presence xmlns=\"urn:ietf:params:xml:ns:pidf\"\n\
-          xmlns:es=\"urn:ietf:params:xml:ns:pidf:status:rpid-status\"\n\
-          entity=\"%s\">\n\
-<tuple id=\"sg89ae\">\n\
-<status>\n\
-<basic>open</basic>\n\
-<es:activities>\n\
-  <es:activity>on-the-phone</es:activity>\n\
-</es:activities>\n\
-</status>\n\
-<contact priority=\"0.8\">%s</contact>\n\
-<note>on the phone</note>\n\
-</tuple>\n\
-</presence>",
-	      from, from);
-    }
-  else if (presence_mode==LINPHONE_STATUS_OUTTOLUNCH)
-    {
-      snprintf(buf, 5000, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\
-<presence xmlns=\"urn:ietf:params:xml:ns:pidf\"\n\
-          xmlns:es=\"urn:ietf:params:xml:ns:pidf:status:rpid-status\"\n\
-          entity=\"%s\">\n\
-<tuple id=\"sg89ae\">\n\
-<status>\n\
-<basic>open</basic>\n\
-<es:activities>\n\
-  <es:activity>meal</es:activity>\n\
-</es:activities>\n\
-</status>\n\
-<contact priority=\"0.8\">%s</contact>\n\
-<note>out to lunch</note>\n\
-</tuple>\n\
-</presence>",
-	      from, from);
-    }
-  else if (presence_mode==LINPHONE_STATUS_OFFLINE)
-    {
-      /* */
-      snprintf(buf, 5000, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\
-<presence xmlns=\"urn:ietf:params:xml:ns:pidf\"\n\
-xmlns:es=\"urn:ietf:params:xml:ns:pidf:status:rpid-status\"\n\
-entity=\"%s\">\n%s",
-	      from,
-"<tuple id=\"sg89ae\">\n\
-<status>\n\
-<basic>closed</basic>\n\
-<es:activities>\n\
-  <es:activity>permanent-absence</e:activity>\n\
-</es:activities>\n\
-</status>\n\
-</tuple>\n\
-\n</presence>\n");
-    }
-
-  i = eXosip_build_publish(&pub, (char *)from, (char *)from, NULL, "presence", "1800", "application/pidf+xml", buf);
-
-  if (i<0)
-    {
-      ms_message("Failed to build publish request.");
-      return -1;
-    }
-
-  eXosip_lock();
-  i = eXosip_publish(pub, from); /* should update the sip-if-match parameter
-				    from sip-etag  from last 200ok of PUBLISH */
-  eXosip_unlock();
-  if (i<0)
-    {
-      ms_message("Failed to send publish request.");
-      return -1;
-    }
-  return 0;
+			       LinphoneOnlineStatus presence_mode){
+	int err;
+	SalOp *op=sal_op_new(proxy->lc->sal);
+	err=sal_publish(op,linphone_proxy_config_get_identity(proxy),
+	    linphone_proxy_config_get_identity(proxy),linphone_online_status_to_sal(presence_mode));
+	sal_op_release(op);
+	return err;
 }
 
+/**
+ * Returns the route set for this proxy configuration.
+**/
+const char *linphone_proxy_config_get_route(const LinphoneProxyConfig *obj){
+	return obj->reg_route;
+}
+
+/**
+ * Returns the SIP identity that belongs to this proxy configuration.
+ *
+ * The SIP identity is a SIP address (Display Name <sip:username@domain> )
+**/
+const char *linphone_proxy_config_get_identity(const LinphoneProxyConfig *obj){
+	return obj->reg_identity;
+}
+
+/**
+ * Returns TRUE if PUBLISH request is enabled for this proxy.
+**/
+bool_t linphone_proxy_config_publish_enabled(const LinphoneProxyConfig *obj){
+	return obj->publish;
+}
+
+/**
+ * Returns the proxy's SIP address.
+**/
+const char *linphone_proxy_config_get_addr(const LinphoneProxyConfig *obj){
+	return obj->reg_proxy;
+}
+
+/**
+ * Returns the duration of registration.
+**/
+int linphone_proxy_config_get_expires(const LinphoneProxyConfig *obj){
+	return obj->expires;
+}
+
+/**
+ * Returns TRUE if registration to the proxy is enabled.
+**/
+bool_t linphone_proxy_config_register_enabled(const LinphoneProxyConfig *obj){
+	return obj->reg_sendregister;
+}
+
+struct _LinphoneCore * linphone_proxy_config_get_core(const LinphoneProxyConfig *obj){
+	return obj->lc;
+}
+
+/**
+ * Add a proxy configuration.
+ * This will start registration on the proxy, if registration is enabled.
+**/
 int linphone_core_add_proxy_config(LinphoneCore *lc, LinphoneProxyConfig *cfg){
 	if (!linphone_proxy_config_check(lc,cfg)) return -1;
+	if (ms_list_find(lc->sip_conf.proxies,cfg)!=NULL){
+		ms_warning("ProxyConfig already entered, ignored.");
+		return 0;
+	}
 	lc->sip_conf.proxies=ms_list_append(lc->sip_conf.proxies,(void *)cfg);
 	linphone_proxy_config_apply(cfg,lc);
 	return 0;
 }
 
-extern void linphone_friend_check_for_removed_proxy(LinphoneFriend *lf, LinphoneProxyConfig *cfg);
-
+/**
+ * Removes a proxy configuration.
+ *
+ * LinphoneCore will then automatically unregister and place the proxy configuration
+ * on a deleted list. For that reason, a removed proxy does NOT need to be freed.
+**/
 void linphone_core_remove_proxy_config(LinphoneCore *lc, LinphoneProxyConfig *cfg){
-	MSList *elem;
 	lc->sip_conf.proxies=ms_list_remove(lc->sip_conf.proxies,(void *)cfg);
 	/* add to the list of destroyed proxies, so that the possible unREGISTER request can succeed authentication */
 	lc->sip_conf.deleted_proxies=ms_list_append(lc->sip_conf.deleted_proxies,(void *)cfg);
@@ -483,13 +475,26 @@ void linphone_core_remove_proxy_config(LinphoneCore *lc, LinphoneProxyConfig *cf
 	if (lc->default_proxy==cfg){
 		lc->default_proxy=NULL;
 	}
-	/* invalidate all references to this proxy in our friend list */
-	for (elem=lc->friends;elem!=NULL;elem=ms_list_next(elem)){
-		linphone_friend_check_for_removed_proxy((LinphoneFriend*)elem->data,cfg);
-	}
-	
 }
-
+/**
+ * Erase all proxies from config.
+ *
+ * @ingroup proxy
+**/
+void linphone_core_clear_proxy_config(LinphoneCore *lc){
+	MSList* list=ms_list_copy(linphone_core_get_proxy_config_list((const LinphoneCore*)lc));
+	for(;list!=NULL;list=list->next){
+		linphone_core_remove_proxy_config(lc,(LinphoneProxyConfig *)list->data);
+	}
+	ms_list_free(list);
+}
+/**
+ * Sets the default proxy.
+ *
+ * This default proxy must be part of the list of already entered LinphoneProxyConfig.
+ * Toggling it as default will make LinphoneCore use the identity associated with
+ * the proxy configuration in all incoming and outgoing calls.
+**/
 void linphone_core_set_default_proxy(LinphoneCore *lc, LinphoneProxyConfig *config){
 	/* check if this proxy is in our list */
 	if (config!=NULL){
@@ -508,6 +513,9 @@ void linphone_core_set_default_proxy_index(LinphoneCore *lc, int index){
 	else linphone_core_set_default_proxy(lc,ms_list_nth_data(lc->sip_conf.proxies,index));
 }
 
+/**
+ * Returns the default proxy configuration, that is the one used to determine the current identity.
+**/
 int linphone_core_get_default_proxy(LinphoneCore *lc, LinphoneProxyConfig **config){
 	int pos=-1;
 	if (config!=NULL) *config=lc->default_proxy;
@@ -517,39 +525,13 @@ int linphone_core_get_default_proxy(LinphoneCore *lc, LinphoneProxyConfig **conf
 	return pos;
 }
 
-static int rid_compare(const void *pcfg,const void *prid){
-	const LinphoneProxyConfig *cfg=(const LinphoneProxyConfig*)pcfg;
-	const int *rid=(const int*)prid;
-	ms_message("cfg= %s, cfg->rid=%i, rid=%i",cfg->reg_proxy, cfg->rid, *rid);
-	return cfg->rid-(*rid);
-}
-
-LinphoneProxyConfig *linphone_core_get_proxy_config_from_rid(LinphoneCore *lc, int rid){
-	MSList *elem=ms_list_find_custom(lc->sip_conf.proxies,rid_compare, &rid);
-	if (elem==NULL){
-		ms_message("linphone_core_get_proxy_config_from_rid: searching in deleted proxies...");
-		elem=ms_list_find_custom(lc->sip_conf.deleted_proxies,rid_compare, &rid);
-	}
-	if (elem==NULL) return NULL;
-	else return (LinphoneProxyConfig*)elem->data;
-}
-
+/**
+ * Returns an unmodifiable list of entered proxy configurations.
+**/
 const MSList *linphone_core_get_proxy_config_list(const LinphoneCore *lc){
 	return lc->sip_conf.proxies;
 }
 
-
-void linphone_proxy_config_process_authentication_failure(LinphoneCore *lc, eXosip_event_t *ev){
-	LinphoneProxyConfig *cfg=linphone_core_get_proxy_config_from_rid(lc, ev->rid);
-	if (cfg){
-		cfg->auth_failures++;
-		/*restart a new register so that the user gets a chance to be prompted for a password*/
-		if (cfg->auth_failures==1){
-			linphone_proxy_config_register(cfg);
-		}
-	}
-}
-
 void linphone_proxy_config_write_to_config_file(LpConfig *config, LinphoneProxyConfig *obj, int index)
 {
 	char key[50];
@@ -574,6 +556,8 @@ void linphone_proxy_config_write_to_config_file(LpConfig *config, LinphoneProxyC
 	lp_config_set_int(config,key,"reg_expires",obj->expires);
 	lp_config_set_int(config,key,"reg_sendregister",obj->reg_sendregister);
 	lp_config_set_int(config,key,"publish",obj->publish);
+	lp_config_set_int(config,key,"dial_escape_plus",obj->dial_escape_plus);
+	lp_config_set_string(config,key,"dial_prefix",obj->dial_prefix);
 }
 
 
@@ -607,6 +591,9 @@ LinphoneProxyConfig *linphone_proxy_config_new_from_config_file(LpConfig *config
 	linphone_proxy_config_enableregister(cfg,lp_config_get_int(config,key,"reg_sendregister",0));
 	
 	linphone_proxy_config_enable_publish(cfg,lp_config_get_int(config,key,"publish",0));
+
+	linphone_proxy_config_set_dial_escape_plus(cfg,lp_config_get_int(config,key,"dial_escape_plus",0));
+	linphone_proxy_config_set_dial_prefix(cfg,lp_config_get_string(config,key,"dial_prefix",NULL));
 	
 	tmp=lp_config_get_string(config,key,"type",NULL);
 	if (tmp!=NULL && strlen(tmp)>0) 
@@ -658,11 +645,13 @@ SipSetup *linphone_proxy_config_get_sip_setup(LinphoneProxyConfig *cfg){
 }
 
 void linphone_proxy_config_update(LinphoneProxyConfig *cfg){
+	LinphoneCore *lc=cfg->lc;
 	if (cfg->commit){
 		if (cfg->type && cfg->ssctx==NULL){
 			linphone_proxy_config_activate_sip_setup(cfg);
 		}
-		linphone_proxy_config_register(cfg);
+		if (lc->sip_conf.register_only_when_network_is_up || lc->network_reachable)
+			linphone_proxy_config_register(cfg);
 		cfg->commit=FALSE;
 	}
 }
@@ -681,6 +670,10 @@ SipSetupContext *linphone_proxy_config_get_sip_setup_context(LinphoneProxyConfig
 	return cfg->ssctx;
 }
 
+/**
+ * @}
+**/
+
 LinphoneAccountCreator *linphone_account_creator_new(struct _LinphoneCore *core, const char *type){
 	LinphoneAccountCreator *obj;
 	LinphoneProxyConfig *cfg;
@@ -755,5 +748,13 @@ void linphone_account_creator_destroy(LinphoneAccountCreator *obj){
 	}
 }
 
+void linphone_proxy_config_set_user_data(LinphoneProxyConfig *cr, void * ud) {
+	cr->user_data=ud;
+}
+
+void * linphone_proxy_config_get_user_data(LinphoneProxyConfig *cr) {
+	return cr->user_data;
+}
+
 
 
