@@ -71,17 +71,19 @@ void linphone_gtk_update_my_contact(GtkWidget *w){
 	const char *username=gtk_entry_get_text(GTK_ENTRY(linphone_gtk_get_widget(pb,"username")));
 	const char *displayname=gtk_entry_get_text(GTK_ENTRY(linphone_gtk_get_widget(pb,"displayname")));
 	int port=linphone_core_get_sip_port(linphone_gtk_get_core());
-	osip_from_t *parsed=linphone_core_get_primary_contact_parsed(linphone_gtk_get_core());
+	LinphoneAddress *parsed=linphone_core_get_primary_contact_parsed(linphone_gtk_get_core());
 	char *contact;
 	g_return_if_fail(parsed!=NULL);
 	if (username[0]=='\0') return;
-	if (port!=5060)
-		contact=g_strdup_printf("%s <sip:%s@%s:%i>",displayname,username,parsed->url->host,port);
-	else
-		contact=g_strdup_printf("%s <sip:%s@%s>",displayname,username,parsed->url->host);
+
+	linphone_address_set_display_name(parsed,displayname);
+	linphone_address_set_username(parsed,username);
+	linphone_address_set_port_int(parsed,port);
+	contact=linphone_address_as_string(parsed);
 	gtk_entry_set_text(GTK_ENTRY(linphone_gtk_get_widget(pb,"sip_address")),contact);
 	linphone_core_set_primary_contact(linphone_gtk_get_core(),contact);
-	g_free(contact);
+	ms_free(contact);
+	linphone_address_destroy(parsed);
 	linphone_gtk_load_identities();
 }
 
@@ -209,7 +211,7 @@ void linphone_gtk_play_ring_file(GtkWidget *w){
 }
 
 void linphone_gtk_echo_cancelation_toggled(GtkWidget *w){
-	linphone_core_enable_echo_cancelation(linphone_gtk_get_core(),
+	linphone_core_enable_echo_cancellation(linphone_gtk_get_core(),
 		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)));
 }
 
@@ -225,6 +227,17 @@ enum {
 	CODEC_NCOLUMNS
 };
 
+static void fmtp_edited(GtkCellRendererText *renderer, gchar *path, gchar *new_text, gpointer userdata){
+	GtkListStore *store=(GtkListStore*)userdata;
+	GtkTreeIter iter;
+	if (gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(store),&iter,path)){
+		PayloadType *pt;
+		gtk_list_store_set(store,&iter,CODEC_PARAMS,new_text,-1);
+		gtk_tree_model_get(GTK_TREE_MODEL(store),&iter,CODEC_PRIVDATA,&pt,-1);
+		payload_type_set_recv_fmtp(pt,new_text);
+	}
+}
+
 static void linphone_gtk_init_codec_list(GtkTreeView *listview){
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
@@ -266,11 +279,14 @@ static void linphone_gtk_init_codec_list(GtkTreeView *listview){
 						"foreground",CODEC_COLOR,
                                                    NULL);
 	gtk_tree_view_append_column (listview, column);
+	renderer = gtk_cell_renderer_text_new ();
 	column = gtk_tree_view_column_new_with_attributes (_("Parameters"),
                                                    renderer,
                                                    "text", CODEC_PARAMS,
 						"foreground",CODEC_COLOR,
+	    				"editable",TRUE,
                                                    NULL);
+	g_signal_connect(G_OBJECT(renderer),"edited",G_CALLBACK(fmtp_edited),store);
 	gtk_tree_view_append_column (listview, column);
 	/* Setup the selection handler */
 	select = gtk_tree_view_get_selection (listview);
@@ -293,7 +309,7 @@ static void linphone_gtk_show_codecs(GtkTreeView *listview, const MSList *codecl
 		gchar *color;
 		const char *params="";
 		struct _PayloadType *pt=(struct _PayloadType *)elem->data;
-		if (payload_type_enabled(pt)) status=_("Enabled");
+		if (linphone_core_payload_type_enabled(linphone_gtk_get_core(),pt)) status=_("Enabled");
 		else status=_("Disabled");
 		if (linphone_core_check_payload_type_usability(linphone_gtk_get_core(),pt)) color="blue";
 		else color="red";
@@ -309,7 +325,7 @@ static void linphone_gtk_show_codecs(GtkTreeView *listview, const MSList *codecl
 					CODEC_PARAMS,params,
 					CODEC_PRIVDATA,(gpointer)pt,
 					CODEC_COLOR,(gpointer)color,
-					CODEC_INFO,(gpointer)payload_type_get_description(pt),
+					CODEC_INFO,(gpointer)linphone_core_get_payload_type_description(linphone_gtk_get_core(),pt),
 					-1);
 	}
 	
@@ -431,7 +447,7 @@ static void linphone_gtk_codec_set_enable(GtkWidget *button, gboolean enabled){
 	if (gtk_tree_selection_get_selected(sel,&mod,&iter)){
 		store=GTK_LIST_STORE(mod);
 		gtk_tree_model_get(mod,&iter,CODEC_PRIVDATA,&pt,-1);
-		payload_type_set_enable(pt,enabled);
+		linphone_core_enable_payload_type(linphone_gtk_get_core(),pt,enabled);
 		gtk_list_store_set(store,&iter,CODEC_STATUS, enabled ? _("Enabled") : _("Disabled"), -1);
 	}
 }
@@ -698,15 +714,39 @@ void linphone_gtk_lang_changed(GtkComboBox *combo){
 	}
 }
 
+static void linphone_gtk_ui_level_adapt(GtkWidget *top) {
+	gboolean ui_advanced;
+	const char *simple_ui = linphone_gtk_get_ui_config("simple_ui", "parameters.codec_tab parameters.transport_frame parameters.ports_frame");
+
+	ui_advanced = linphone_gtk_get_ui_config_int("advanced_ui", TRUE);
+	if (ui_advanced) {
+		linphone_gtk_visibility_set(simple_ui, "parameters", top, TRUE);
+	} else {
+		linphone_gtk_visibility_set(simple_ui, "parameters", top, FALSE);
+	}
+}
+
+void linphone_gtk_ui_level_toggled(GtkWidget *w) {
+	gint ui_advanced;
+	GtkWidget *top;
+
+	top = gtk_widget_get_toplevel(w);
+	ui_advanced = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
+	linphone_gtk_set_ui_config_int("advanced_ui", ui_advanced);
+	linphone_gtk_ui_level_adapt(top);
+}
+
 void linphone_gtk_show_parameters(void){
 	GtkWidget *pb=linphone_gtk_create_window("parameters");
 	LinphoneCore *lc=linphone_gtk_get_core();
 	const char **sound_devices=linphone_core_get_sound_devices(lc);
 	const char *tmp;
-	osip_from_t *contact;
+	LinphoneAddress *contact;
 	LinphoneFirewallPolicy pol;
 	GtkWidget *codec_list=linphone_gtk_get_widget(pb,"codec_list");
 	int mtu;
+	int ui_advanced;
+
 	/* NETWORK CONFIG */
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(pb,"ipv6_enabled")),
 				linphone_core_ipv6_enabled(lc));
@@ -755,18 +795,19 @@ void linphone_gtk_show_parameters(void){
 					linphone_core_get_video_device(lc),CAP_IGNORE);
 	linphone_gtk_fill_video_sizes(linphone_gtk_get_widget(pb,"video_size"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(pb,"echo_cancelation")),
-					linphone_core_echo_cancelation_enabled(lc));
+					linphone_core_echo_cancellation_enabled(lc));
 
 	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(linphone_gtk_get_widget(pb,"ring_chooser")),
 					linphone_core_get_ring(lc));
 	/* SIP CONFIG */
 	contact=linphone_core_get_primary_contact_parsed(lc);
 	if (contact){
-		if (contact->displayname) 
-			gtk_entry_set_text(GTK_ENTRY(linphone_gtk_get_widget(pb,"displayname")),contact->displayname);
-		if (contact->url->username)
-			gtk_entry_set_text(GTK_ENTRY(linphone_gtk_get_widget(pb,"username")),contact->url->username);
+		if (linphone_address_get_display_name(contact)) 
+			gtk_entry_set_text(GTK_ENTRY(linphone_gtk_get_widget(pb,"displayname")),linphone_address_get_display_name(contact));
+		if (linphone_address_get_username(contact))
+			gtk_entry_set_text(GTK_ENTRY(linphone_gtk_get_widget(pb,"username")),linphone_address_get_username(contact));
 	}
+	linphone_address_destroy(contact);
 	linphone_gtk_show_sip_accounts(pb);
 	/* CODECS CONFIG */
 	linphone_gtk_init_codec_list(GTK_TREE_VIEW(codec_list));
@@ -776,6 +817,13 @@ void linphone_gtk_show_parameters(void){
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(linphone_gtk_get_widget(pb,"upload_bw")),
 				linphone_core_get_upload_bandwidth(lc));
 
+	/* UI CONFIG */
 	linphone_gtk_fill_langs(pb);
+	ui_advanced = linphone_gtk_get_ui_config_int("advanced_ui", 1);
+	linphone_gtk_set_ui_config_int("advanced_ui", ui_advanced);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(pb,"ui_level")),
+				ui_advanced);
+	linphone_gtk_ui_level_adapt(pb);
+
 	gtk_widget_show(pb);
 }
